package org.lsst.gruth

import org.codehaus.groovy.control.CompilerConfiguration

/**
 * for each test file :
 * <UL>
 * <LI> creates a list of <TT>GroupSpec</TT> : each one defines a batch of test set.
 * Each "test set" is a <TT>RawTestSpec</TT>
 * that may define a test that will be run with different combination of values.
 * So when this test set is "expanded" it will generate many Test runs (<TT>TztRun</TT>).
 * <LI>
 *  Once collected these <TT>BatchSpec</TT> are to be expanded, then run, then tests reports are to be generated.
 *  Normally these operations are performed once all the <TT>BatchSpecs</TT> have been collected
 *  (the file describing the test has been read).
 * <LI>
 *  There are three categories of <TT>BatchSpecs</TT> :
 *  <UL>
 *      <LI> <TT>BuildGroupSpec</TT> are there to generate objects (constructors, factories).
 *      The corresponding tests are to be run BEFORE the others.
 *      It could be specified (<TT>_run</TT> command) to expand and run these tests right after
 *      their specification has been read (this is because they may generate variables used
 *      by other tests specifications).
 *      <LI> <TT>InvokeGroupSpec</TT> are there to invoke methods (static of instance methods).
 *      The corresponding tests are to be run AFTER tests on constructors and factories.
 *      The order of execution is to be randomized (except if explicit <TT>_run</TT> is called
 *      in the code).
 *      <LI> <TT>FreeSpec</TT> is groovy code to be executed after all the preceding ones.
 *      it has access to reporting and do whatever the programmer sees fit.
 *
 *  </UL>
 *
 * </UL>
 *
 * <P>
 *  This code has bindings that are accessible through different gru scripts read .
 *  This enables to chain tests: objects generated by one script can be used by another script.
 *  (TODO: this is bad for memory, how can we register/read these Objects if they are not Serializable?)
 *  <P/>
 *  System property <TT>gru.dumpSource</TT> make gru print its source code before running it.
 *
 * User: bear
 * Date: 15/06/11
 */

if (args) {
    def conf = new CompilerConfiguration()
    conf.setScriptBaseClass("org.lsst.gruth.grutils.GruScript")
    def shell = new GroovyShell(conf)
    //ClassLoader cloader = ClassLoader.getSystemClassLoader()
    args.each {
        //  check if not ".gru" already
        def str = it
        if(str.endsWith('.gru')) {
            str = str.subSequence(0, str.lastIndexOf('.'))
        }
        // transforms it to an URL ending with .gru
         str = '/' + str.replace('.' as char, '/' as char)
        int lastIndex = str.lastIndexOf('/')
        def scriptName = str.substring(lastIndex +1 )
        str = str + '.gru'
        InputStream is = this.class.getResourceAsStream(str)
        //is = org.lsst.gruth.grutils.TaggedsMap.class.getResourceAsStream(str)

        //is = cloader.getResourceAsStream(str)
        if (is == null) { // then reads a file
            try {
                File file = new File(it)
                if (!file.exists()) {
                    file = new File(it + ".gru")
                }
                is = new FileInputStream(file)

            } catch (IOException exc) {
                //was documented as possible!!!! (implicit log)
                //log.severe ( "$str or $it not found in class path" )
                println "$str or $it not found in class path or file path"
            }

        }
        if (is != null) {
            InputStream endIS = new ByteArrayInputStream('\n_runAll() ; \n'.getBytes());
            InputStream allIs = new SequenceInputStream(is, endIS)

            boolean debug = Boolean.getBoolean('gru.dumpSource')
            if (debug) {
                BufferedReader bufr = new BufferedReader(new InputStreamReader(allIs))
                String line
                while (null != (line = bufr.readLine())) {
                    println line
                }
            }
            System.setProperty("script.name", scriptName)
            shell.setVariable('_scriptName', scriptName)
            shell.setProperty('_scriptName', scriptName)
            shell.evaluate(allIs)
            //shell.evaluate('_runAll()')
        }
        // generate the tests request by combinations of parameters

        // these should be done once all the tests are gathered
        // randomize the order of execution ?

        // execute each tests

        // pass the reports to reportHandler in alphabetical order
        // should be run at the end and not here
    }

}
else {
    println "Usage: gru script_canonical_name_or_filename ..."
}
