package org.lsst.gruth.grutils

import java.util.regex.Pattern

/**
 * Created initially by a call to _with**, it deals with a group of test
 * specification. In the
 * end will be expanded in a map of tests to execute and report.
 *
 * @author bamade
 */
//todo make it abstract
//TODO modify so as to have a run/report option for each test (enabling millions of tests)
class GroupSpec {

    /**
    * type of specification. see <TT>SpecType</TT>.
     * not all combination are valid for subclasses (for instance InvokeGroupSpec cannot be CTOR)
    */
    SpecType specType;
    /**
     * Name for group (either explicit or generated)
     */
    String groupName;
    /**
     * name of Class: may be null if objects of different types are tested
     */
    String className;
    /**
     * the Class: may be null if objects of different types are tested
     */
    Class clazz;
    /**
     * name of method
     */
    String methodName;
    /**
     * for constructors or static methods only (may differ if there are many objects).
     */
    def execCode;// constructor or static method
    /**
     * specific to tests with instances.
     * (TODO: move to InvokeGrouSpecs if possible)
     */
    List<TaggedObject> instances = []
    /**
     * code to be executed before running all tests in this group
     */
    Closure setUpCode;
    /**
     * code to be executed after the tests have been run
     */
    Closure tearDownCode;
    /**
     * the list of tests specification to be populated
     */
    List<RawTestSpec> rawSpecs = [];
    /**
     * a template for the TztReport that will be generated.
     * this report can be modified by events during set up time (while building the specifications of tests)
     * then will be cloned for every <TT>TztRun</TT> before it is run and
     * then the report will be completed.
     */
    TztReport reportTemplate;
    /**
     * a copy of the _this global Binding
     */
    Binding _this
    /**
     * A binding specific to this group.
     */
    Binding _thisG = new Binding()




    GroupSpec(SpecType type, Binding bind, String className, String methodName) {
        this.specType = type
        _this = bind
        this.className = className
        this.methodName = methodName
        reportTemplate = new TztReport(className, methodName)
    }

    GroupSpec(SpecType type, Binding bind, String className) {
        this(type, bind, className, '<ctor>')
    }

    GroupSpec(SpecType type, Binding bind) {
        this(type, bind, '#dynamic', '#dynamic')
    }

    /**
     * This method should be abstract : seed docs in subclasses.
     * It shoud gather : setUpCode, tearDownCode, and all the RawTestSpec
     * @param clos
     * @return
     */
    GroupRun _group(Closure clos) {
        // generate the tests request by combinations of parameters

        // these should be done once all the tests are gathered
        // randomize the order of execution ?

        // execute each tests

        // pass the reports to reportHandler in alphabetical order
        // should be run at the end and not here
    }

    /**
     * code to be run once the tests of a group have been run
     * (kind of teartDown code)
     * @param clos
     * @return
     */
    GroupRun _post(Closure clos) {
        this.tearDownCode = clos
        return this
    }

    //TODO: write a _pre method

    void _pre(Closure clos) {
        this.setUpCode = clos ;
    }

    /**
     * Prepares a RawTestSpec structure.
     * @param map : argument to be passed to the rawTestSpec structure.
     * @return
     */
    RawTestSpec _test(Map map) {
        RawTestSpec rawSpecs = new RawTestSpec()
        this.rawSpecs << rawSpecs
        rawSpecs.rawArgs = map
        // for each key will generate
        return rawSpecs
    }

    /**
     *  Enables a simplified syntax
     *  instead of <TT>_test map _xpect { / expectations/ } </TT>
     *  one can write <TT>_test map { / expectations/ } </TT>
     * @param map
     * @param clos
     * @return
     */
    //TODO: test syntax

    RawTestSpec _test (Map map, Closure clos){
        return _test(map)._xpect(clos)
    }

    /**
     * Once the _group information has been collected, each _group code
     * should end by calling this method.
     * A <TT>GroupRun</TT> structure will be built to prepare the execution of tests.
     * Since the tests specification uses set of values the tests to be run should
     * be generated by combining values.
     * <BR>
     *  This method is extremely complex and should be carefully  maintained.
     * @return
     */
    GroupRun _prepare() {
        GroupRun res = new GroupRun()
        // TODO: create constructor
        res.groupName = groupName;
        res.specType = this.specType
        res.setUpCode = setUpCode
        res.tearDownCode = tearDownCode
        res._this = _this
        // register the Run
        //println "GROUP " + groupName
        def map = _this.groups
        map[specType].put(groupName, res)

        rawSpecs.each {
            def models = fromRawArgsToRawRuns(it.rawArgs, reportTemplate, _thisG)
            // now complete each model with rawPsecs info
            // only names are in TztRun
            RawTestSpec specs = it
            models.each {
                // we copy information from specs
                it.preCode = specs.setUp
                it.postCode = specs.tearDown
                it.expectations = specs.expectations
                //it.specs = this
                // might have been already set up ...
                if (reportTemplate.caught.size() != 0) {
                    it.exceptionFired = true
                }
            }
            res.list += models

        }
        // if there are many objects we ought to multiply
        if (instances.size() == 0) {
            res.list.each {
                // in each TztRun we have
                // we have only names and codes (set, tear, xpect)
                // now we need codes to be executed and their groupName
                it.thatClazz = clazz
                it.thatExecCode = execCode
                // TODO: there should be the groupName of class and method
                //println 'each '+ it._argsNames

            }
        } else {
            List<TztRun> newRunList = []
            //dealing with many objects
            // if execode is null change it
            //TODO: test the type of operation
            instances.each {
                TaggedObject tagged = it
                Object curObj = tagged.object
                Class clazzObj = curObj.getClass()
                Throwable th = null;
                def execCode = null ;
                try {
                    execCode = curObj.&"$methodName"

                } catch (Exception exc) {
                    th= exc
                }
                res.list.each {
                    TztRun newRun = it.clone()
                    newRun.thatClazz = clazzObj
                    newRun.thatExecCode = execCode
                    String clazzName =clazzObj.getSimpleName()
                    newRun._className =  clazzName
                    // duplicates
                    newRun._report.className = clazzName
                    newRun._report.supportingObject = tagged
                    newRun._it = tagged
                    if(th!= null){
                        newRun._report.caught << th
                        newRun._report.rawDiagnostic = RawDiagnostic.TOOL_OR_CODE_ERROR
                    }
                    newRunList << newRun

                }
            }

            res.list = newRunList

        }
        // TODO: generate the test groupName (if one test just gets the groupName otherwise create )
        // do that earlier
        return res


    }
/*.
    def _run() {
        this._run()
    }
    */

    static Pattern NULL_PATTERN = Pattern.compile('null.*')
    /**
     * each element of the Map looks like KEY: arg1,ar2, arg3 KEY2: argn, argm
     * KEY will become the  "rootName" of a test run
     * then each arg will be exploded:
     * <UL>
     *     <LI>
     *    - arg can be a TaggedsMap
     *    <LI>
     *    - NO: arg can be a Map -> NamedMap (NO won' tests other objects that are maps!)
     *    <LI>
     *    - arg can be an TaggedObject -> NamedMap create
     *    <LI>
     *    - arg can be an WithStringKey -> TaggedObject -> namedsMap
     *    <LI>
     *    - NO: arg can be an ordinary object -> toString ->TaggedObject -> namedsMap (No refuse!)
     *    now we have list of namedMap for each KEY
     *
     * <UL>
     * better have a map which is itself a TaggedsMap
     *
     * each TztRun is only populated with :
     *  -rootName
     *  -report.testName
     */
    List<TztRun> fromRawArgsToRawRuns(Map map, TztReport template, Binding groupBind) {
        List<TztRun> res = []
        map.each {
            def localRunList = []
            String key = it.key
            TztRun model = new TztRun(template.clone(), _this, groupBind)
            //TODO : duplicates
            model._rootName = key
            model._report.testName = key
            model._methodName = this.methodName
            // DONE model._report.methodName = this.methodName
            // if value is a single object put it in an array
            def value = it.value
            if (!(value instanceof List)) {
                value = [value]
            }
            def array = value as List
            // we check for nature of each member of the list
            // namedsMap or WithStringKey -> else error in code
            // now we have a list of N parameters that are all TaggedsMap
            int size = array.size()
            for (int idx = 0; idx < size; idx++) {
                def val = array[idx]
                if (val == null) {
                    model.reportException(new MissingDataException(), RawDiagnostic.TOOL_OR_CODE_ERROR)
                    // TODO: DO SOMETHING HERE otherwise the code will crash!
                }
                else if (val instanceof TaggedsMap) {
                    def dict = val as TaggedsMap
                    if (dict.size() == 0) {
                        model.reportException(new MissingDataException(), RawDiagnostic.MISSING_DATA)
                    }
                    //println "pssed taggedMap"
                    //ok
                } else if (val instanceof TaggedObject) {
                    TaggedObject tobj = val as TaggedObject
                    // if value is null and tag not starts with null -> missing data
                    if (tobj.object == null) {
                        //if (!NULL_PATTERN.matcher(tobj.key).matches()) {
                        String keyObj = tobj.key
                        int keyLength = keyObj.length()
                        if (!keyObj.substring(0, keyLength > 3 ? 4 : keyLength).equalsIgnoreCase('null')) {
                            model.reportException(new MissingDataException(tobj.key), RawDiagnostic.MISSING_DATA)
                        }
                    }

                    array[idx] = new TaggedsMap(tobj)
                }

                else if (val instanceof WithStringKey) {
                    array[idx] = new TaggedsMap(val as WithStringKey)
                } else {
                    //@TODO: accept objects and tag them
                    //model.reportException(new UnTaggedObjectException(val), RawDiagnostic.WARNINGS)
                    array[idx] = new TaggedsMap(val)
                }

            }// for
            // we create a combiner list
            Combiner comb = null;
            for (int ix = array.size() - 1; ix >= 0; ix--) {
                comb = new Combiner(comb, array[ix] as TaggedsMap);
            }
            localRunList << model

            comb.populate(localRunList, 0, model)
            // we tes if model is alone
            // TODO why this code initially? it causes bug in naming ?
            //if (localRunList.size() == 1) {
                //model.useRootNameOnly = true
            //}

            res += localRunList

        }

        return res

    }

    /**
    *  A utility class tom help build combinations of tests.
    */
    static class Combiner {
        Combiner next
        TaggedsMap map



        Combiner(Combiner nxt, TaggedsMap mp) {
            next = nxt
            map = mp
        }

        def populate(List array, int index, TztRun current) {
            TztRun otherRun = current.clone()
            Iterator iterator = map.values().iterator()
            if(array.size() == 0 ) {
                throw new IllegalArgumentException ("empty arguments specification")
            }
            // first one
            TaggedObject taggedObj = iterator.next()
            current._argsNames[index] = taggedObj.key
            current._args[index] = taggedObj.object
            //println 'call populate: ' + taggedObj.key + ' ' + taggedObj.object
            if (next != null) {
                next.populate(array, index + 1, current)
            } else {
                // println current._argsNames
            }
            // others
            while (iterator.hasNext()) {
                // println 'after next'
                TztRun elseRun = current.clone()
                array << elseRun
                taggedObj = iterator.next()
                elseRun._argsNames[index] = taggedObj.key
                elseRun._args[index] = taggedObj.object

                //TODO: not efficient: done already!!! should go in descending order
                if (next != null) {
                    next.populate(array, index + 1, elseRun)
                }
            }

        }
    }


}
