package org.lsst.gruth.grutils

/**
 * A <TT>GroupSpec</TT> for  testing instance methods.
 * @author bamade
 *
 */
class InvokeGroupSpec extends GroupSpec {
    boolean methodSpecified = false
    TaggedsMap tmap = new TaggedsMap()

    /**
     *
     * @param bind
     * @param objs a list of tagged objects on which tests will be performed
     */
    InvokeGroupSpec(Binding bind, Object... objs) {
        super(SpecType.INSTANCE_METHOD, bind)
        //toDo: if list empty !!! report error
        // populates the  list
        objs.each {
            if (it instanceof TaggedsMap || it instanceof TaggedObject) {
                tmap += it
            } else {
                this.reportTemplate.caught << new UnTaggedObjectException(it)
                this.reportTemplate.rawDiagnostic = RawDiagnostic.TOOL_OR_CODE_ERROR
            }
        }
    }

    /**
     * GRU DSL method  to specify the method to test on the objects.
     * @param methodName
     * @return
     */
    InvokeGroupSpec _method(String methodName) {
        // TODO: check if called once! use methodSpecified
        this.methodName = methodName;
        this.reportTemplate.methodName = methodName
        methodSpecified = true;
        // todo: problem reporting if ony one of these fails
        // just now all fail!!!
        try {
            tmap.values().each {
                TaggedObject current = it
                this.instances << it
            }

        } catch (GroovyRuntimeException err) {
            this.reportTemplate.caught << err
            this.reportTemplate.rawDiagnostic = RawDiagnostic.NOT_YET_IMPLEMENTED

        } catch (Exception err) {
            //Cst._LOGGER.log(Level.INFO,"",err)
            this.reportTemplate.caught << err
            this.reportTemplate.rawDiagnostic = RawDiagnostic.TOOL_OR_CODE_ERROR

        }
        return this
    }

    /**
     * Enables a simplified syntax: so instead of having
     * <TT> _method 'methodName' _group { /test group /} </TT>
     * one can specify
     * <TT> _method 'methodName' { /test group /} </TT>
     * @param methodName
     * @param clos
     * @return
     */
    GroupRun _method(String methodName, Closure clos) {
        return _method(methodName)._group(clos)
    }
 /**
    * calls the other _group method by generating a name for the group
    * @param clos
    * @return
    */
    GroupRun _group(Closure clos) {
        int number = _this.groupNumber
        String name = "#group${number}"
        _this.groupNumber = number+1
        return _group(name, clos)
    }
    /**
    *  prepares a set of test (GroupRun) (for instances method only for the moment -no instance factories-)
    * @param name
    * @param clos
    * @return
    */
    GroupRun _group(String name, Closure clos) {
        this.groupName = name
        if (!methodSpecified) {
            this.reportTemplate.caught << new OutOfContextException("calling group without specifying method")
            this.reportTemplate.rawDiagnostic = RawDiagnostic.TOOL_OR_CODE_ERROR
        }
        try {
            clos.setDelegate(this)
            clos.setBinding(_this)
            _this.currentGroup = this
            // gathers the requested tests
            clos.call()
            _this.currentGroup = null
        } catch (Throwable err) {
            this.reportTemplate.caught << err
            this.reportTemplate.rawDiagnostic = RawDiagnostic.TOOL_OR_CODE_ERROR
        }
        //println 'reportTemplate:' + this.reportTemplate+ ' execCode ' + execCode

        return this._prepare()
    }
}
