package org.lsst.gruth.grutils

import java.lang.reflect.Method

/**
 * An Object linked to its key.
 * Used to store internally objects in TaggedsMaps.
 * looks like a Map.Entry but used internally for specific purposes in GRU
 */
class TaggedObject<V> implements WithStringKey, Serializable {
    public static final long serialVersionUID = 1237L;
    final String key;

    /**
     * beware if not Serializable this object will dumped on serialization
     */
    transient V object;
    //??? don't remember???
    boolean keep;

    TaggedObject(String key, V object) {
        this.key = key
        this.object = object
    }

    TaggedObject(Map.Entry<String, V> entry) {
        this.key = entry.key
        this.object = entry.value

    }

    private void writeObject(java.io.ObjectOutputStream out) throws IOException {
        out.defaultWriteObject()
        if (!(this.object instanceof Serializable)) {
            out.writeObject('-not serializable-:' + this.object.toString())
        } else {
            out.writeObject(this.object)
        }
    }

    private void readObject(java.io.ObjectInputStream input) throws IOException, ClassNotFoundException {
        input.defaultReadObject()
        this.object = input.readObject()
    }

    boolean equals(o) {
        if (this.is(o)) return true;
        if (!(o instanceof TaggedObject)) return false;

        TaggedObject that = (TaggedObject) o;

        if (key != that.key) return false;

        return true;
    }

    int hashCode() {
        return key.hashCode();
    }

    String toString() {
        key + ': ' + object
    }

    String getKey() {
        return key
    }

    public static TaggedObject tag(Object obj) {
        if(obj == null) {
            return new TaggedObject("NULL_gen", null)
        }
        if (obj instanceof TaggedObject) return obj
        if (obj instanceof WithStringKey) {
            WithStringKey keyed = obj as WithStringKey
            return new TaggedObject(keyed.key, keyed)
        }
        /*
        if(obj.metaClass.respondsTo(obj, 'getName')){
            return new TaggedObject(obj.getName(), obj)
        }
        */
        Class clazz = obj.class
        def methNames = ["getName", "getKey", "getId"]
        for (String name: methNames) {
            try {
                Method method = clazz.getMethod(name)
                String st = String.valueOf(method.invoke(obj))
                return new TaggedObject(st, obj)

            } catch (Exception exc) {
                // sorry
            }
        }

        String projectedTag = obj.toString()
        return new TaggedObject(projectedTag, obj)
    }


}
