package org.lsst.gruth.grutils

/**
 * The final structure describing a test and allowing to run a test.
 * @author bamade
 *
 */
class TztRun implements Runnable, Cloneable {
    /**
     * Name of class (duplicated in _report)
     */
    String _className;

    /**
     * the current class
     */
    Class thatClazz;
    //duplicated in _report
    /**
     * name of method ( special name "&lt;build&gt;" for constructors
     */
    String _methodName;
    /**
     * Code to be executed (method, constructor) ( a methodClosure)
     */
    def thatExecCode;
    //duplicated in _report
    /**
     * This is the root String from which the name ot the test (and of the variable
     * built by constructors and factories) will be tagged.
     * If there is only one test tagged with this name the test will retain this tag
     * (see variable <TT>useRootNameOnly</TT> if there are many tests run built along
     * the same _rootName the name of the test in the corresponding TztReport
     * will be built by concatenating the rootName with tags from arguments (+ some shortening process)
     */
    // TODO: IMPORTANT generate the tztReport Name correctly!!!!
    String _rootName;
    boolean useRootNameOnly = false;
    /**
     * the names of the arguments passed to the test (remember those are tagged)
     */
    def _argsNames = []
    /**
     * the arguments to the tests
     */
    def _args = [];
    // no change because xpect has a special status
    // only ifcaught is tolerated
    //TODO change to RawTestSpec
    Closure preCode
    Closure expectations
    Closure postCode

    //created and modified
    /**
     * data generated by the call to method or constructor (if any)
     */
    def _result;
    // the object on which the call of method was performed
    /**
     * the current object on which we operate (for methods only)
     */
    def _it;
    boolean exceptionFired;
    //TODO: dump initialisation of this field
    TztReport _report = new TztReport()
    // all variables not in scope should be prefixed by _thisL !
    //Binding _thisL = new Binding()
    Binding _this
    //TODO: probably useless since execution of code does not work properly
    Binding _thisG

    /**
     *  local context shared by preCode, postCode and expectations
     */
    Binding _thisL = new Binding()


    public TztRun(TztReport template, Binding bindScript, Binding bindGroup) {
        _report = template
        _this = bindScript
        _thisG = bindGroup
    }


    TztRun clone() {
        TztRun res = super.clone()
        res._report = _report.clone()
        res._args = _args.clone()
        res._argsNames = _argsNames.clone()
        //res._thisL = new Binding()
        //todo: check for others
        return res
    }

    def get_it() {
        return this._it
    }

    def get_currentObject() {
        return this._it.object
    }

    def get_currentKey() {
        return this._it.key
    }

    def _message(String mess) {
        this._report._message(mess)
    }
    /**
     *  used to add some data to the report (does not change the result of the test)
     *
     * @param data any serializable data
     * @return the argument
     */
    Serializable _reportData(Serializable data) {
        return this._report._reportData(data)
    }

    def _neutral(String stringExpression) {
        this._report._neutral(stringExpression)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    def _neutral(String message, Closure closure) {
        this._report._neutral(message, closure)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    def _doNotReport() {
        this._report._doNotReport()
    }

    def _okIfCaught(Class<? extends Throwable> throwClass) {
        this._report._okIfCaught(throwClass)
    }

    //TODO: change that not called
    void reportException(Throwable th, RawDiagnostic result) {
        this._report.reportException(th, result)
        this.exceptionFired = true
    }

    /**
     * Adds an <TT>AssertionReport</TT> to the current <TT>TztReport</TT> object:
     * the result is <TT>FAILED</TT> if the evaluation of the string is true
     * (the report also contains the evaluated string).
     * [NOT WORKING PROPERLY -YET-]
     * @param stringBooleanExpr a Gstring representing a valid Groovy expression
     * @return
     */
    def _failIf(String stringBooleanExpr) {
        this._report._failIf(stringBooleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }
    /**
     * Adds an <TT>AssertionReport</TT> to the current <TT>TztReport</TT> object:
     * the result is <TT>FAILED</TT> if the evaluation of the expression is true
     * (the report also contains the message)
     * @param message to be displayed in the report
     * @param booleanExpr before evaluating exceptionfired should be tested
     * @return
     */
    def _failIf(String message, boolean booleanExpr) {
        this._report._failIf(message, booleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    /**
     * same specs as failIf but faisl if false
     * @param stringBooleanExpr
     * @return
     */
    def _failIfNot(String stringBooleanExpr) {
        this._report._failIfNot(stringBooleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    /**
     * same specs as failIf but fails if false
     */
    def _failIfNot(String message, boolean booleanExpr) {
        this._report._failIfNot(message, booleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    /**
     * Adds an <TT>AssertionReport</TT> to the current <TT>TztReport</TT> object:
     * the result is <TT>WARNINGS</TT> if the evaluation of the string is true
     * (the report also contains the evaluated string).
     * [NOT WORKING PROPERLY -YET-]
     * @param stringBooleanExpr a Gstring representing a valid Groovy expression
     * @return
     */
    def _warnIf(String stringBooleanExpr) {
        this._report._warnIf(stringBooleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    /**
     * Adds an <TT>AssertionReport</TT> to the current <TT>TztReport</TT> object:
     * the result is <TT>WARNINGS</TT> if the evaluation of the expression is true
     * (the report also contains the message)
     * @param message to be displayed in the report
     * @param booleanExpr
     * @return
     */
    def _warnIf(String message, boolean booleanExpr) {
        this._report._warnIf(message, booleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    /**
     * same specs as warnIf but warns if false
     */
    def _warnIfNot(String stringBooleanExpr) {
        this._report._warnIfNot(stringBooleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }

    /**
     * same specs as warnIf but warns if false
     */
    def _warnIfNot(String message, boolean booleanExpr) {
        this._report._warnIfNot(message, booleanExpr)
        //TODO : horrible! make this a return value of methods
        this.exceptionFired = this._report.exceptionFired
    }
    /**
     * runs the test
     */
    void run() {
        _this.currentRun = this
        normalizeName()
        //if there are errors does not run just generate reports
        // run the closure in context
        // at each stage if an exception is generated a boolean is set
        // generates report and registers it
        // if exception fired
        // TODO: not coherent exception fired and list of exceptions in report change!
        if (this._report.caught.size() == 0) {
            boolean getGoing = true;
            // TODO :does not work!!!
            //if (this.preCode) this.preCode.delegate(this)
            if (this.preCode) {
                try {
                    preCode.call()

                } catch (Exception exc) {
                    reportException(exc, RawDiagnostic.TOOL_OR_CODE_ERROR)
                    getGoing = false
                }
            }
            if (getGoing) {
                try {
                    Object[] args = this._args as Object[]
                    // STUPID GROOY thing!!!!
                    if (this.thatExecCode != null) {
                        //todo: change for testing of operation type!
                        if (_it != null) {

                            if (args.size() == 1) {
                                // @TODO CHANGE FOR NO_ARG specific name
                                if (args[0] == []) {
                                    //println ' EMPTY '
                                    this._result = this.thatExecCode.call()
                                }
                                else {
                                    //println ' ARGS ' + args
                                    this._result = this.thatExecCode args[0]
                                }
                            } else {
                                this._result = this.thatExecCode.call(args)
                            }
                        } else {
                            // @TODO CHANGE FOR NO_ARG specific name
                            if (args[0] == []) {
                                //println ' EMPTY '
                                this._result = this.thatExecCode.call()
                            } else {
                                this._result = this.thatExecCode args
                            }
                            //sets _it
                            _this.setVariable('_it', _result)
                        }
                        this._report.data = _result
                        this._report.rawDiagnostic = RawDiagnostic.SUCCESS
                    }
                    //println this._rootName + ' = ' + this._result
                } catch (GroovyRuntimeException grex) {
                    //println 'GROOVY EXC'
                    reportException(grex, RawDiagnostic.NOT_YET_IMPLEMENTED)
                } catch (Throwable th) {
                    reportException(th, RawDiagnostic.FAILED)

                }
            }
            if (getGoing && this.expectations) {
                //TODO: ERROR object is not an instance of declaring class
                //this.expectations.delegate(this)
                this.expectations.setBinding(_this)
                try {
                    this.expectations.call()
                } catch (Throwable th) {
                    reportException(th, RawDiagnostic.TOOL_OR_CODE_ERROR)
                }

            }

            // call postCode code if error -> ??? log something
            // TODO :does not work!!!
            //if (this.postCode) this.postCode.delegate(this)
            if (getGoing && this.postCode) {
                try {
                    postCode.call()
                } catch (Exception exc) {
                    reportException(exc, RawDiagnostic.WARNINGS)
                }
            }

            _report.gatherResult()

        } else {
            //println this._report.caught
        }
        _this.currentRun = null
    }

    private void normalizeName() {
        if (!useRootNameOnly) {
            String name = "$_rootName $_argsNames"
            this._report.testName = name
        }
    }
}
