package org.lsst.gruth.nodes

import org.lsst.gruth.grutils.PossibleConstructorOrSetterProblem
import org.lsst.gruth.grutils.Tracer
import org.lsst.gruth.jutils.ComponentNode.IndirectHash
import org.lsst.gruth.jutils.MapArgs.LEntry
import org.lsst.gruth.jutils.*

/**
 *
 * @author bamade
 *
 */
class ComponentFactory {
    ComponentNode topNode;

    //TODO: deprecate: incoherent
    ComponentFactory(ComponentNode topNode) {
        this.topNode = topNode
    }

    ComponentFactory() {}

    @Deprecated
    ComponentNode build() {
        Map<String, Object> registry;
        //TODO: stupid this is null!
        return buildFrom(registry, topNode);
    }
    ComponentNode buildObjectTreeFrom(ComponentNode node) {
        IndirectMap<String, Object> map = new IndirectHash<String, Object>()
        ComponentNode res = buildFrom(map, node)
        res.orphan = false;
        res.setRoot(true)
        return res
    }

    ComponentNode buildFrom(IndirectMap<String, Object> registry, ComponentNode node) {
        //start from children
        ArrayList<ComponentNode> newChildren = null;
        ArrayList<ComponentNode> children = node.getChildren()
        if (children != null) {
            int nbChildren = children.size()
            if (nbChildren > 0) {
                newChildren = new ArrayList<ComponentNode>(nbChildren)
                for (ComponentNode child: children) {
                    newChildren.add(buildFrom(registry, child))
                }
            }
        }
        // gets the key
        String key = node.getKey()
        // gets the value as a Class
        Object value = node.getRealValue()
        //System.out.println("key" + key + " value:" +value);
        Class clazz = null;
        if (value instanceof LEntry && ((LEntry)value).value instanceof ImplPlaceHolder) {
            ImplPlaceHolder placeHolder = ((ImplPlaceHolder) ((LEntry)value).value)
            if (!placeHolder.isSet()) {
                throw new EmptyParmException(placeHolder)
            }
            clazz = placeHolder.value
        } else if (value instanceof Class) {
            clazz = (Class) value;
        } else {
            throw new IncompatibleTypeException(" $key node value not a class: value = $value attrib: ${node.getAttributes()}"
                    , Class, value != null ? value.getClass() : Void.TYPE)
        }
        // gets the parameters of constructor
        Object actualNodeValue = null;
        Map rawArgs = node.getAttributes()
        if (rawArgs instanceof MapArgs) {
            MapArgs originArgs = (MapArgs) rawArgs
            boolean hasNullConstructor = true ;
            try  {
                clazz.getConstructor()
            } catch (NoSuchMethodException exc){
                hasNullConstructor = false ;
            }
            if ( originArgs.isSafe()) {
                //TODO: change this test in case of having arguments that are not modifiable
                // TODO: major bug!
                // is ValuesOnly is not proper here!!!
                if ((!hasNullConstructor) || originArgs.isValuesOnly()) {
                    List listArgs = originArgs.asFullList(registry)
                    // builds the object value
                    //println "doing constructor call $clazz"
                    try {
                        Tracer.trace(Tracer.NODE_EVAL, key, "calling constructor for", clazz, "with args :", listArgs)
                        actualNodeValue = clazz.metaClass.invokeConstructor(listArgs as Object[])
                    } catch( Exception exc) {
                        throw new PossibleConstructorOrSetterProblem("", exc, clazz.toString(), listArgs)
                    }
                } else {
                    MapArgs args = originArgs.asFullMap(registry)
                    // builds the object value
                    //println "class instanciation : "+ clazz
                    def execCode = clazz.metaClass.&invokeConstructor
                    try {
                        //println clazz.toString()+ " " + String.valueOf(args)
                        Tracer.trace(Tracer.NODE_EVAL, key, "calling no-args constuctor for", clazz,"with setters for :", args)
                        actualNodeValue = execCode args
                    } catch( Exception exc) {
                        throw new PossibleConstructorOrSetterProblem("", exc, clazz.toString(), args)
                    }
                }
            } else {
                throw new EmptyParmException("arguments list with empty values")
            }
        } else if (rawArgs == null) {
            Tracer.trace(Tracer.NODE_EVAL, key, "calling no-arg constructor for :", clazz)
            def execCode = clazz.metaClass.&invokeConstructor
            try {
                actualNodeValue = execCode.call()
            } catch( Exception exc) {
                throw new PossibleConstructorOrSetterProblem("", exc, clazz.toString())
            }
        }
        //TODO : else
        // builds the object value
        // builds a new Component Node
        ComponentNode res = ComponentNode.createOrphan(registry, key, actualNodeValue)
        // register children
        if (newChildren != null) {
            res.addOrphans(registry, newChildren)
        }
        // now go to calls Node
        if( node.calls != null) {
            res.calls = buildCalls(registry, node)
        }
        return res

    }

    ComponentNode buildCalls(IndirectMap<String, Object> registry, ComponentNode node) {
        ComponentNode callsNode =  node.getCalls();
        if(callsNode != null) {
            for( ComponentNode child : callsNode.getChildren()) {
                Map map = child.getAttributes() ;
                if( map != null && map instanceof MapArgs) {
                    MapArgs args = (MapArgs) map ;
                    if(! args.isSafe()) {
                        throw new EmptyParmException("method ${child.getName()} arguments list with empty values")
                    }
                }
            }
        }
        return callsNode ;
    }
}
