package org.lsst.gruth.ggre;

import org.lsst.gruth.grutils.Advice;
import org.lsst.gruth.grutils.AnnotatedReport;
import org.lsst.gruth.grutils.TztReport;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

/**
 * @author Alain Becker
 */
// Date: 30/08/12

@SuppressWarnings("serial")
public class ReportDetailPanel
        extends JTabbedPane
        implements Observer {


    private class ReportPanel extends JPanel {
        private int fieldCount = 0;
        private JTextField txtKey;
        private JTextField txtAdvice;
        private JTextField txtPostComment;
        private JTextField txtRawResult;
        private JTextField txtName;
        private JTextField txtClass;
        private JTextField txtMethod;

        private JTextField txtData;
        private JTextField txtAdditionalData;

        private JTextField txtMessages;
        private JTextField txtAssertions;
        private JTextField txtCaughts;

        public ReportPanel() {
            init();
        }

        public void init() {

            GridBagLayout gbl = new GridBagLayout();
            //TODO: CHECK THIS 4 lines
            gbl.columnWidths = new int[]{0};
            gbl.rowHeights = new int[]{0};
            gbl.columnWeights = new double[]{Double.MIN_VALUE};
            gbl.rowWeights = new double[]{Double.MIN_VALUE};
            this.setLayout(gbl);

            manageLabelsAndFields(new JLabel("Key\u21E8object:"), txtKey = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Advice:"), txtAdvice = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Review comment:"), txtPostComment = new JTextField(), true);
            //TODO: put here a mark for problems  -> green/red
            manageLabelsAndFields(new JLabel("    "), null, false);
            manageLabelsAndFields(new JLabel("RawResults:"), txtRawResult = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Name:"), txtName = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Class:"), txtClass = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Method:"), txtMethod = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Data:"), txtData = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Add. data:"), txtAdditionalData = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Messages:"), txtMessages = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Assertions:"), txtAssertions = new JTextField(), false);
            manageLabelsAndFields(new JLabel("Caughts:"), txtCaughts = new JTextField(), false);

        }
        /**
         * Utility method to handle repetitive aspects of the UI creation - aimed at the grid population
         *
         * @param txt
         */
        private void manageLabelsAndFields(JComponent lbl, JTextField txt, boolean editable) {

            // Create the layout constraints object
            GridBagConstraints c = new GridBagConstraints();
            c.anchor = GridBagConstraints.NORTHEAST;
            c.insets = new Insets(0, 0, 5, 5);
            c.gridx = 0;
            c.gridy = fieldCount;
            c.insets = new Insets(0, 0, 5, 5);

            // Add label to the grid
            this.add(lbl, c);

            // TextFields common graphical aspects
            if (txt != null) {
                txt.setEditable(editable);
                txt.setColumns(30);
                txt.setAlignmentY(Component.BOTTOM_ALIGNMENT);
                txt.setAlignmentX(Component.LEFT_ALIGNMENT);
                txt.setHorizontalAlignment(SwingConstants.LEADING); // .LEFT -- no visible result... :-//
                if (!editable) {
                    txt.setBackground(Color.LIGHT_GRAY);
                }

                // Slight differences in the layout contraints for the TextField
                c.gridx = 1;
                c.anchor = GridBagConstraints.NORTHWEST;

                // Add TextField to the grid
                this.add(txt, c);
            }

            // update row count
            fieldCount++;
        }
        /**
         * Updates textfields contents (and formated report zone) from given {@link TztReport}, which can be <code>null.</code>
         * <p/>
         * If TztReport is null, textfields are replaced by given default contents
         *
         * @param aReport         The report from which to pull and display fields, or null to empty them.
         * @param defaultContents The string to use to empty fields if <code>report==null</code><br />
         *                        use <code>""</code> to actually empty the fields,<br />
         *                        or <code>null</code> to get default boilerplate contents.
         */
        private void populateFields(AnnotatedReport aReport, String defaultContents) {

            if (defaultContents == null) {
                defaultContents = "";
            }

            if (aReport != null) {
                TztReport report = aReport.getReport();

                String supporting = report.getSupportingObjectString();

                // Text fields
                txtKey.setText(report.getUniqueKey());
                txtRawResult.setText(report.getRawDiagnostic().toString());

                /**/
                Advice advice = aReport.getAdvice();
                txtAdvice.setText((advice != null) ? advice.toString() : "");
                /**/
                txtName.setText(report.getTestName());
                txtClass.setText(report.getClassName());
                txtMethod.setText(report.getMethodName());
                txtPostComment.setText(
                        (aReport.getUserComment() != null)
                                ? aReport.getUserComment()
                                : ""
                );
                txtData.setText(
                        (report.getData() != null)
                                ? report.getData().toString()
                                : ""
                );
                txtAdditionalData.setText(
                        (report.getAdditionalData() != null)
                                ? report.getAdditionalData().toString()
                                : ""
                );
                txtMessages.setText(String.valueOf(report.getMessages()));
                txtAssertions.setText( String.valueOf( report.getListAssertions()));
                txtCaughts.setText(String.valueOf(report.getCaught()));

            } else { // If report is null, set default values

                txtKey.setText(defaultContents);
                txtRawResult.setText(defaultContents);
                //txtAdvice.setText(defaultContents);
                txtName.setText(defaultContents);
                txtClass.setText(defaultContents);
                txtMethod.setText(defaultContents);
                txtPostComment.setText(defaultContents);
                txtData.setText(defaultContents);
                txtAdditionalData.setText(defaultContents);
                txtMessages.setText(defaultContents);
                txtAssertions.setText(defaultContents);
                txtCaughts.setText(defaultContents);
            }
        }

        /**
         * Idem {@link populateFields(AnnotatedReport, String)}, but uses default contents if <code>report==null</code>
         *
         * @param aReport The report from which to pull and display fields, or null to empty them
         */
        private void populateFields(AnnotatedReport aReport) {
            populateFields(aReport, null);
        }

    }



    private AnnotatedReport aReport;
    private ReportPanel mainPanel;
    private ReportPanel previousPanel ;


    public ReportDetailPanel() {
        //this.add(contentPanel, BorderLayout.BEFORE_FIRST_LINE);
        mainPanel = new ReportPanel() ;

      mainPanel.txtPostComment.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                if (aReport != null) {
                    aReport.setUserComment(mainPanel.txtPostComment.getText());
                }
            }
        });

        previousPanel = new ReportPanel() ;
        previousPanel.txtPostComment.setEnabled(false);

        this.addTab("current Report", mainPanel);
        this.addTab("                        ", previousPanel);

    }



    /**
     * Observer interface impl.
     */
    @Override
    public void update(Observable o, Object notification) {
        //System.out.println("ReportDetailPanel received push notification of ReportPanel selection change.");
        if (notification == null) {
           mainPanel.populateFields(null);
           previousPanel.populateFields(null);
           mainPanel.txtAdvice.setToolTipText(null);

        } else if (notification instanceof ReportsPanelNotification) {
            ReportsPanelNotification notif = (ReportsPanelNotification) notification;
            this.aReport = notif.getEditedReport();
            setSelectedComponent(mainPanel);
           mainPanel.populateFields(this.aReport);
            mainPanel.txtAdvice.setToolTipText(
                    (notif.isDifferenceNotification())
                            ? ("Original advice was " + notif.getOriginalReport().getAdvice())
                            : null
            );
            if(aReport.getNeededPreviousReport() != null) {
                previousPanel.setEnabled(true);
                previousPanel.populateFields(aReport.getNeededPreviousReport());
                setTitleAt(1, "previous report version :" +aReport.getNeededPreviousVersion());
            } else {
                previousPanel.setEnabled(false);
                previousPanel.populateFields(null);
                setTitleAt(1, "                   ");
            }
        } else { // shouldnt' reach here //TODO: decide wether we have better do nothing or throw a RTE
            previousPanel.setEnabled(false);
            mainPanel.populateFields(null);
            previousPanel.populateFields(null);
            mainPanel.txtAdvice.setToolTipText(null);
            throw new RuntimeException("ReportDetailPanel Observer received a " + notification.getClass() + " ; a ReportsPanelNotification (or null) was expected.");

        }
    }


}
