package org.lsst.gruth.ggre;

import org.lsst.gruth.ggre.dialogs.DynDialog;
import org.lsst.gruth.reports.StoreAgent;

import javax.swing.*;
import javax.swing.plaf.BorderUIResource;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.util.Observable;
import java.util.Observer;

/**
 * @author Alain Becker
 * @author bamade
 */
// Date: 30/08/12

public class ReportEditorPanel extends JPanel implements Observer {
    private static final String APP_TITLE_BASE = "GGRE"; // GGRE is a GRU Report Editor ;-)
    private static final String STATUS_HTML_TEMPLATE_BEFORE = "<html><body style='font-family: arial, helvetica, sans-serif;'><div style='padding: 3px;'>";
    private static final String STATUS_HTML_TEMPLATE_AFTER = "</div></body></html>";

    private static final ImageIcon icnOpen = new ImageIcon(ReportEditorPanel.class.getResource("resources/icons/Folder_table.png"));
    private static final ImageIcon icnSave = new ImageIcon(ReportEditorPanel.class.getResource("resources/icons/Database_save.png"));
    //private static final ImageIcon icnClose = new ImageIcon(ReportEditorPanel.class.getResource("resources/icons/cross.png"));

    private String workingDirectoryName = null;
    private StoreAgent store ;


    private String bundleName = null;


    private JButton btnReportViewAsXml;
    private ReportsPanel pnlReports;
    private ReportDetailPanel pnlDetail;
    private JEditorPane edtStatus;
    private DynDialog confirmDialog;

    public ReportEditorPanel(StoreAgent store) {
        this.store = store ;
        initLayout();
    }

    private void initLayout() {
        this.shouldUpdateTitle();
        this.setLayout(new BorderLayout(3,3)); // default layout manager

        Box topBox = Box.createVerticalBox();
        this.add(topBox, BorderLayout.NORTH);

        Box leftBox = Box.createHorizontalBox();
        this.add(leftBox, BorderLayout.WEST);

       // Box middleBox = Box.createHorizontalBox();
        //this.add(middleBox, BorderLayout.CENTER);

        //Box rightBox = Box.createHorizontalBox();
        //this.add(rightBox, BorderLayout.EAST);

        Box bottomBox = Box.createVerticalBox();
        this.add(bottomBox, BorderLayout.SOUTH);

        /********** Menu bar **********/

        JMenuBar menuBar = new JMenuBar();
        menuBar.setAlignmentX(Component.LEFT_ALIGNMENT);
        topBox.add(menuBar, BorderLayout.NORTH);

        Box buttonBox = Box.createHorizontalBox();
        buttonBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        topBox.add(buttonBox);

        /********** Button bar(s) **********/

        JToolBar toolBarFile = new JToolBar("Reports");
        toolBarFile.setRollover(true);
        toolBarFile.setFloatable(false);
        toolBarFile.setAlignmentX(Component.LEFT_ALIGNMENT);
        buttonBox.add(toolBarFile);

        JToolBar toolBarReport = new JToolBar("Report");
        toolBarReport.setRollover(true);
        toolBarReport.setFloatable(false);
        toolBarReport.setAlignmentX(Component.LEFT_ALIGNMENT);
        buttonBox.add(toolBarReport);
        /********** Menu + Button bar (auto) population(items/actions) **********/

        /* A REVOIR*/
        JMenu mnuStore = new JMenu("Store");
        menuBar.add(mnuStore);

        for (StoreActions action: StoreActions.values()) {
            mnuStore.add(new JMenuItem(new StoreAction(action)));
            toolBarFile.add(new JButton(new StoreAction(action)));
        }
       /**/

        /********** Reports table **********/

        pnlReports = new ReportsPanel();
        this.add(pnlReports, BorderLayout.CENTER);
        //middleBox.add(pnlReports);
        //: horrible HACK! to get the proper dimension!!!!
        //int preferredSize = pnlReports.getPossibleWidth() ;
        //Box.Filler filler = new Box.Filler(new Dimension(preferredSize,1),
                //new Dimension(preferredSize,1), new Dimension(4000,1)) ;
        //JPanel fixedW = new JPanel(new BorderLayout()) ;
        //fixedW.add(pnlReports, BorderLayout.CENTER) ;
        //fixedW.add(filler, BorderLayout.SOUTH) ;
        //this.add(fixedW, BorderLayout.CENTER);

        /********** Reports details **********/

        pnlDetail = new ReportDetailPanel();
        //rightBox.add(pnlDetail);
        this.add(pnlDetail, BorderLayout.AFTER_LINE_ENDS);

        /********** Status bar **********/

        edtStatus = new JEditorPane("text/html", null);
        this.populateStatus("No report selected.");
        edtStatus.setEditable(false);
        edtStatus.setVisible(true);
        edtStatus.setBorder(BorderUIResource.getEtchedBorderUIResource());
        bottomBox.add(edtStatus);

        pnlReports.getObservable().addObserver(this); // a test observer
        pnlReports.getObservable().addObserver(pnlDetail); // a functional observer

    }

    /**
     * Utility method used to populate the status
     */
    private void populateStatus(String htmlContents) {
        StringBuilder sb = new StringBuilder(STATUS_HTML_TEMPLATE_BEFORE);
        sb.append(htmlContents);
        sb.append(STATUS_HTML_TEMPLATE_AFTER);
        this.edtStatus.setText(sb.toString());
        // see edtStatus.getEditorKit()
        // and edtStatus.getDocument().
    }


    /**
     */
    private void shouldUpdateTitle() {
        //frame title
    }


    public void update(Observable observable, Object notification) {
        // txtDebug.setText(arg.toString());
        //System.out.println("App received push notification of ReportPanel selection change.");
        if (notification instanceof ReportsPanelNotification) { // sanity check

            ReportsPanelNotification notif = (ReportsPanelNotification) notification;

            StringBuilder sb = new StringBuilder();
            sb.append("Test \"<b>");
            sb.append(notif.getEditedReport().getReport().getUniqueKey());
            sb.append("</b>\"");
            if (notif.isDifferenceNotification()) {
                sb.append(" - original Advice was <b>");
                sb.append(notif.getOriginalReport().getAdvice());
                sb.append("</b>");
            }
            this.populateStatus(sb.toString());


        } else {

            this.populateStatus("No report selected.");

        }

    }
    public String getBundleName() {
        return bundleName;
    }

    public void setBundleName(String bundleName, String version) {
        this.bundleName = bundleName;
        try {
            if(version == null){
                store.setBundle(bundleName);
            } else {
                int versionNumber = Integer.parseInt(version) ;
                store.setBundle(bundleName, versionNumber);
            }
            pnlReports.setStore(store);
            String title = bundleName + " " + store.getVersion() ;
            Window top = SwingUtilities.getWindowAncestor(this);
            if(top instanceof  JFrame){
                Frame ftop = (JFrame) top ;
                ftop.setTitle( "EDITING REPORTS FOR: " + title);
            }
        } catch (Exception exc) {
            //TODO : catch
            JOptionPane.showMessageDialog(this, bundleName + " : no such bundle (" +exc + ")",
                    "Store exception", JOptionPane.ERROR_MESSAGE);
        }
    }

    ///////////////////////////// ACTIONS
    ////// ---------- STORE
    private enum StoreActions {
        OPEN("StoreActionOpen", "Open", "Opens a new bundle", icnOpen),
        //SAVE_AS("StoreActionSave", "Save", "Saves the current report set", icnSave),
        SAVE("StoreActionSave", "Save", "Saves the current report set", icnSave),
        //CLOSE("fileActionClose", "Close", "Closes the current report", icnClose),
        //Quit("fileActionQuit", "Quit", "Quit GGRE", icnClose);
        ;
        private String actionCommand;
        private String label;
        private String description;
        private ImageIcon icon;
        private StoreActions(String actionCommand, String label, String description, ImageIcon icon) {
            this.actionCommand = actionCommand;
            this.label = label;
            this.description = description;
            this.icon = icon;
        }
    }

    /**
     * General-purpose Action aimed at all file-related ones.
     *
     * Its listener will dispatch to the proper method.
     *
     */
    private class StoreAction extends AbstractAction {

        private StoreAction(StoreActions action) {
            putValue("org.lsst.gruth.ggre_storeActionsItem", action);
            putValue(NAME, action.label);  // => component.setText()
            putValue(SHORT_DESCRIPTION, action.description);
            putValue(SMALL_ICON, action.icon);
            putValue(ACTION_COMMAND_KEY, action.actionCommand); // => components.setActionCommand()
        }

        public void actionPerformed(ActionEvent actionEvent) { // ActionListener implementation
            // System.out.println("FileAction.actionPerformed - begins");
            switch ((StoreActions) this.getValue("org.lsst.gruth.ggre_storeActionsItem")) {
                case OPEN: ReportEditorPanel.this.storeActionOpen(actionEvent); break;
                case SAVE: ReportEditorPanel.this.storeActionSave(actionEvent); break;
                //case SAVE_AS: App.this.fileActionSaveAs(actionEvent); break;
                //case CLOSE:  App.this.fileActionClose(actionEvent); break;
                //default: App.this.fileActionNotImplemented(actionEvent);
            }
        }
    }

    ////// OPEN BUNDLE

    public void storeActionOpen(ActionEvent evt) {
        String name = JOptionPane.showInputDialog(this,
                "choose the bundle to open : type name of name:version", "Store content", JOptionPane.QUESTION_MESSAGE) ;
        if(name == null) return ;
        String[] names = name.split(":") ;

        String bundleName = null ;
        String version = null ;
        switch(names.length) {
            case 0 : return ;
            case 2 :  version = names[1] ;
            case 1: bundleName = names[0] ;
        }
        doOpenStoreAndUpdateView(bundleName, version);
    }

    public void doOpenStoreAndUpdateView(String bundleName, String version) {
        setBundleName(bundleName, version);
        pnlReports.showReport();
    }
    /////  SAVE

    public void storeActionSave(ActionEvent evt){
        try {
            store.save();
        } catch (Exception exc) {
            JOptionPane.showMessageDialog(this, bundleName + " : save (" +exc + ")",
                    "Store exception", JOptionPane.ERROR_MESSAGE);
        }

    }
    ///////////////// NAVIGATION DANS un BUNDLE ???
}
