package org.lsst.gruth.ggre;

import org.lsst.gruth.grutils.RawDiagnostic;
import org.lsst.gruth.reports.StoreAgent;
import org.lsst.gruth.grutils.Advice;
import org.lsst.gruth.grutils.AnnotatedReport;
import org.lsst.gruth.grutils.Modification;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Observable;

/**
 * @author Alain Becker
 * @author bamade
 */
// Date: 30/08/12

public class ReportsPanel
        extends JPanel {

    private static final int MARK_COLUMN_INDEX = 0;
    private static final int KEY_COLUMN_INDEX = MARK_COLUMN_INDEX + 1;
    private static final int RAWRESULT_COLUMN_INDEX = KEY_COLUMN_INDEX + 1;
    private static final int ADVICE_COLUMN_INDEX = RAWRESULT_COLUMN_INDEX + 1;
    // Code assumes ADVICE_COLUMN_INDEX is last column, so please keep COLUMN_COUNT initialized as ADVICE_COLUMN_INDEX + 1
    private static final int COLUMN_COUNT = ADVICE_COLUMN_INDEX + 1; // DO NOT CHANGE THIS!

    private StoreAgent store;
    private JTable table;
    private JScrollPane scroll ;
    private ReportsTableTableModel tm;
    //TODO : reevaluate : contrary to memory model!
    private java.util.List<AnnotatedReport> originalCopyReports;

    private static Color defaultTableSelectionBackground;
    private static Color defaultTableBackground;

    private JComboBox cbo;

    private int possibleWidth ;

    private Observable observable = new ReportsPanelObservable();

    Observable getObservable() {
        return this.observable;
    }

    public int getPossibleWidth() {
        return possibleWidth;
    }

    private ReportsPanelObservable getPrivateObservable() {
        return (ReportsPanelObservable) this.observable;
    }

    public StoreAgent getStore() {
        return store;
    }

    public void setStore(StoreAgent store) {
        this.store = store;
        tm.setStore(store);
        int size = store.size();
        //System.out.println("taille store" + size);
        originalCopyReports = new ArrayList<AnnotatedReport>(size);
        for (int ix = 0; ix < size; ix++) {
            originalCopyReports.add(null);
        }
        //System.out.println(" taille originale" + originalCopyReports.size());
    }

    public ReportsPanel(StoreAgent store) {
        this();
        setStore(store);
    }

    public ReportsPanel() {
        initPLAFwise();
        this.setLayout(new BorderLayout());

        // Our custom TableModel
        tm = new ReportsTableTableModel();
        // TableColumnModel tcm = new DefaultTableColumnModel();
        ListSelectionModel lsm = new DefaultListSelectionModel();
        lsm.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        // Initialize the ComboBox that will be used by our custom TableCellEditor
        cbo = new JComboBox();
        // The combo should only be populated with non-mark Advice values
        for (Advice advice : Advice.values()) {
            //if (advice.isValue()) {
                cbo.addItem(advice);
            //}
        }
        // table = new JTable(tm, null, lsm);
        table = new ReportsTable(tm, null, lsm);

        // table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);


        // Our custom selectionListener
        table.getSelectionModel().addListSelectionListener(new ReportsTableSelectionListener());

        // Our custom renderer for the Advice column
        // will be unhooked after table population... Will be hooked after that.
        // table.getColumnModel().getColumn(2).setCellRenderer(new ReportsTableAdviceRenderer());
        TableCellEditor editor = new DefaultCellEditor(cbo);
        table.getColumnModel().getColumn(ADVICE_COLUMN_INDEX).setCellEditor(editor);
        ReportsTableAdviceRenderer renderer = new ReportsTableAdviceRenderer();
        for (Enumeration<TableColumn> columns = table.getColumnModel().getColumns(); columns.hasMoreElements(); ) { // Enumerations are not Iterable :-(
            columns.nextElement().setCellRenderer(renderer);
        }
        // SIZE MATTERS!
        int maxSize = 0 ;
        FontMetrics fontMetrics = table.getFontMetrics(table.getFont()) ;
        TableColumn col = table.getColumnModel().getColumn(MARK_COLUMN_INDEX);
        int width = 4 ;
        width = sizeOf(fontMetrics, width) ;
        maxSize += width ;
        col.setPreferredWidth(width);
        col.setMaxWidth(width);
        col.setMinWidth(width);
        col.setWidth(width);
        maxSize += width ;
        col = table.getColumnModel().getColumn(KEY_COLUMN_INDEX);
        width = 50 ;
        width = sizeOf(fontMetrics, width) ;
        col.setPreferredWidth(width);
        col.setMinWidth(width);
        col.setWidth(width);
        maxSize += width ;
        col = table.getColumnModel().getColumn(RAWRESULT_COLUMN_INDEX);
        width = 20 ;
        width = sizeOf(fontMetrics, width) ;
        col.setPreferredWidth(width);
        col.setMinWidth(width);
        col.setWidth(width);
        maxSize += width ;
        //col.setMaxWidth(width);
        col = table.getColumnModel().getColumn(ADVICE_COLUMN_INDEX);
        width = 40 ;
        width = sizeOf(fontMetrics, width) ;
        col.setPreferredWidth(width);
        col.setMinWidth(width);
        col.setWidth(width);
        maxSize += width ;
        possibleWidth = maxSize +4 ;

        JPanel panel = new JPanel(new BorderLayout()) ;
        panel.add(table) ;
        //JScrollPane scroll = new JScrollPane(panel);
        scroll = new JScrollPane(table);
        //scroll.setViewportView(table);
        this.add(scroll);
        //this.add(table) ;
    }

    private String strOfLength(int length) {
        StringBuilder stb = new StringBuilder() ;
        for(int ix = 0 ; ix < length; ix++) {
            stb.append('-') ;
        }
        return stb.toString();
    }

    private int sizeOf(FontMetrics metric, int length) {
        String st = strOfLength(length) ;
        return metric.stringWidth(st);
    }

    private void initPLAFwise() {

        defaultTableSelectionBackground = javax.swing.UIManager.getDefaults().getColor("Table.selectionBackground");
        if (defaultTableSelectionBackground == null) {
            defaultTableSelectionBackground = Color.BLUE;
        }

        defaultTableBackground = javax.swing.UIManager.getDefaults().getColor("Table.background");
        if (defaultTableBackground == null) {
            defaultTableBackground = Color.WHITE;
        }

        // See also:
        // ? javax.swing.plaf.ColorUIResource.getColor("Table.selectionBackground", Color.GREEN)
        // : javax.swing.plaf.ColorUIResource.getColor("Table.background", Color.GRAY)
    }

    @SuppressWarnings("serial")
    private class ReportsTableTableModel extends AbstractTableModel {

        StoreAgent store;

        public ReportsTableTableModel(StoreAgent store) {
            this.store = store;
        }

        public ReportsTableTableModel() {

        }

        public StoreAgent getStore() {
            return store;
        }

        public void setStore(StoreAgent store) {
            this.store = store;
        }

        /**
         * "Advice" is the only editable column
         */
        @Override
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return (columnIndex < ADVICE_COLUMN_INDEX) ? false : true; // (Advice is last column) is assumed here
        }

        public Object getValueAt(int row, int column) {
            if (store == null) return null;
            AnnotatedReport aReport = store.get(row);
            if (aReport == null) return null;
            switch (column) {
                case MARK_COLUMN_INDEX:
                    Modification modif = aReport.getModification();
                    if (modif == null) return "";
                    return modif.getSimplifiedNotation();
                case KEY_COLUMN_INDEX:
                    return aReport.getReport().getUniqueKey();
                case RAWRESULT_COLUMN_INDEX:
                    return aReport.getReport().getRawDiagnostic();
                case ADVICE_COLUMN_INDEX:
                    return aReport.getAdvice();
                default:
                    return null;

            }
        }

        @Override
        public Class<?> getColumnClass(int columnIndex) {
            switch (columnIndex) {
                case MARK_COLUMN_INDEX:
                    return String.class;
                case KEY_COLUMN_INDEX:
                    return String.class;
                case RAWRESULT_COLUMN_INDEX:
                    return RawDiagnostic.class;
                case ADVICE_COLUMN_INDEX:
                    return Advice.class;
                default:
                    return null;
            }
        }

        public int getRowCount() {
            if (store == null) return 0;
            return store.size();
        }

        public int getColumnCount() {
            return COLUMN_COUNT;
        }

        @Override
        public String getColumnName(int columnIndex) {
            switch (columnIndex) {
                case MARK_COLUMN_INDEX:
                    return "";
                case KEY_COLUMN_INDEX:
                    return "Key\u21E8object";
                case RAWRESULT_COLUMN_INDEX:
                    return "RawDiagnostic";
                case ADVICE_COLUMN_INDEX:
                    return "Advice";
                default:
                    return null;
            }
        }


    }

    /**
     * Custom {@link JTable} that overrides its {@link JTable#editingStopped(ChangeEvent)} implementation,
     * in order to intercept {@link Advice} modifications and propagate them
     * to the underlying <code>ArrayList&lt;TztReport&gt;</code> model,
     * and fire report change to our <code>Observers</code>, before returning to JTable code and default behaviour.
     */

    @SuppressWarnings({"serial"})
    private class ReportsTable extends JTable {

        public ReportsTable(TableModel tm, TableColumnModel tcm, ListSelectionModel lsm) {
            super(tm, tcm, lsm);


        }

        // TODO: as this is now implemented is (* extends JTable), we have now easyer means to access edited cell:
        // e.g.: this.editingColumn , this.editingRow
        @Override
        public void editingStopped(ChangeEvent e) { // CellEditorListener impl.
            TableCellEditor editor = getCellEditor();
            if ((editor != null) && (editor.getCellEditorValue() instanceof Advice)) { // we are interested in Advice editions only
                Advice editorAdvice = (Advice) editor.getCellEditorValue();
                Advice tableAdvice = (Advice) table.getValueAt(editingRow, ADVICE_COLUMN_INDEX);

                //System.out.println("ReportsTable implements AdviceEditorListener.editingStopped() triggered at row #" + editingRow + " - found the follownig values:");
                //System.out.println(" - in table: " + tableAdvice);
                //System.out.println(" - in editor: " + editorAdvice);

                if (!editorAdvice.equals(tableAdvice)) { // editingStopped on an actual new value
                    //System.out.println(" => NEW VALUE; will be propagated to model; we will trigger Observers as well");
                    synchronized (ReportsPanel.this) { // We don't want our reports list to get changed again while we propagate its... changes :-)
                        //ReportsPanel.this.editedReports.get(editingRow).setAdvice(editorAdvice);
                        AnnotatedReport aReport = store.get(editingRow);
                        aReport.userModification(editorAdvice);
                        //store.get(editingRow).setAdvice(editorAdvice);
                        ReportsPanelObservable obs = ReportsPanel.this.getPrivateObservable();
                        obs.sendNotification(editingRow);
                    }
                } else { // false positive (like initial mouse invokation of the combo editor)
                    System.out.println(" => false positive");
                }
            }
            super.editingStopped(e);
        }

        @Override
        public boolean getScrollableTracksViewportWidth(){
            if(getParent() instanceof  JViewport) {
                JViewport viewport = (JViewport) getParent();
                return viewport.getWidth() > this.getPreferredSize().width ;
            }
                return true ;

        }

        @Override
        public Dimension getPreferredScrollableViewportSize() {
            return this.getPreferredSize() ;
        }

    }

    /**
     * Extending Observable provides us with access to the protected (setChanged/clearChanged) methods
     */
    class ReportsPanelObservable extends Observable { // This is NOT boilerplate. Do NOT remove.

        /* TODO UNCOMMENT */
        public ReportsPanelNotification buildNotification(int index, Boolean differenceNotification) {
            final AnnotatedReport editedReport = store.get(index);
            // trace

            AnnotatedReport originalReport = null;
            try {
                originalReport = originalCopyReports.get(index);
            } catch (IndexOutOfBoundsException exc) {
                //TODO :
            }
            if (originalReport == null) {
                originalReport = editedReport.clone();
                //originalCopyReports.ensureCapacity(index+1);
                originalCopyReports.set(index, originalReport);
                // first time "edited" is NOT edited!!!! so is the original!!!
                if (editedReport.getModification() == null) {
                    //TODO
                    RawDiagnostic diag =  editedReport.getReport().getRawDiagnostic() ;
                    if(diag.compareTo(RawDiagnostic.OK_MARK) < 0) {

                        editedReport.setModification(Modification.ACKNOWLEDGED_NEGATIVE) ;
                    } else {
                        editedReport.setModification(Modification.ACKNOWLEDGED_POSITIVE) ;
                    }
                    editedReport.setModificationOrigin(AnnotatedReport.Origin.USER);
                }
            }
            // TODO: memory model !!!
            return new ReportsPanelNotification(originalReport, editedReport, index, differenceNotification);
        }

        public ReportsPanelNotification buildNotification(int index) {
            return this.buildNotification(index, null);
        }


        public void sendNotification(int index) {
            setChanged();
            if (index == -1) {
                notifyObservers(null);
            } else {
                ReportsPanelNotification message = buildNotification(index);
                notifyObservers(message);
            }
        }

    }

    @SuppressWarnings("serial")
    private class ReportsTableAdviceRenderer extends DefaultTableCellRenderer {

        public ReportsTableAdviceRenderer() {
            super();
        }

        /**
         * Paint in yellow all cells in a row whose Advice has been edited<br />
         * (implemented as : differs from original value, so "edited then reverted" advices are considered as NOT having been edited)
         * <p/>
         * <p>Unedited/reverted rows are not considered, and retain default behaviour</p>
         * <p>This method is called each time a cell in a column using this renderer needs to be rendered.</p>
         */
        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) { // TableCellRenderer impl.
            Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

            /*
               System.out.println("getTableCellRendererComponent triggered");
               System.out.println("getTableCellRendererComponent - comparing table value '" + table.getValueAt(row, ADVICE_COLUMN_INDEX) + "' typed " +table.getValueAt(row, ADVICE_COLUMN_INDEX).getClass().getCanonicalName());
               System.out.println("getTableCellRendererComponent - with originalReports value '" + ReportsPanel.this.originalReports.get(row).getAdvice()  + "' typed " + ReportsPanel.this.originalReports.get(row).getAdvice().getClass().getCanonicalName());
               */

            // On this row, has the Advice been edited? (or not reverted to its original value)
            c.setBackground((isSelected)
                    ? ReportsPanel.defaultTableSelectionBackground
                    : ReportsPanel.defaultTableBackground
            );
            AnnotatedReport annotatedReport = originalCopyReports.get(row);
            // if null -> not edited
            if (annotatedReport != null) {
                //id different color changes
                //TODO/ beware equals may not be exact!
                if (!annotatedReport.equals(store.get(row))) {
                    c.setBackground((isSelected)
                            ? Color.ORANGE
                            : Color.YELLOW
                    );
                }

                /* does not work properly! but MouseMotionListener is too slow!
                if (c instanceof JComponent) {
                    JComponent jComponent = (JComponent) c;
                    Modification modif = annotatedReport.getModification();
                    jComponent.setToolTipText("<html> "+ modif + "<BR>" + modif.getDetails());
                }
                */
            }

            return c;
        }


//		IMPORTANT NOTE from the DefaultTableCellRenderer JavaDoc :
//
//		As the renderer is only parented for the lifetime of a painting operation
//		we similarly want to avoid the overhead associated with walking the hierarchy for painting operations.
//		So this class overrides the validate, invalidate, revalidate, repaint, and firePropertyChange methods to be no-ops
//		and override the isOpaque method solely to improve performance.
//
//		If you write your own renderer, please keep this performance consideration in mind.

    }


    private class ReportsTableSelectionListener implements ListSelectionListener {

        private int previousIndex = -1;

        /**
         * Reacts on selection changed by propagating the event to registered observers of
         * the ReportPanel.
         * <p/>
         * Due to mouse events firing valueChanged both on MouseDown AND MouseUp,
         * this method checks whether selection <em>actually</em> changed before action.
         *
         * @category ReportListSelectionChanged
         * see http://docs.oracle.com/javase/tutorial/uiswing/events/listselectionlistener.html
         */
        @Override
        public void valueChanged(ListSelectionEvent e) { // ListSelectionListener impl.

            final int newIndex = ((ListSelectionModel) e.getSource())
                    .getMaxSelectionIndex(); // Returns the selected index, or -1 if the selection is empty.

            if (newIndex != this.previousIndex) { // Actual selection change detected : fire Observer(s).update()

                synchronized (ReportsPanel.this) { // we may want consistency

                    this.previousIndex = newIndex;
                    // ReportsPanelObservable obs = ReportsPanel.this.getPrivateObservable();
                    getPrivateObservable().sendNotification(newIndex);

                }
            }

        }

    }

    public void showReport() {
        tm.fireTableDataChanged();
    }


}
