package org.lsst.gruth.ggre.dialogs;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

/**
 * @author  Alain Becker
 * @author bamade
 */
// Date: 30/08/12

    //TODO  a revoir DynDialog
@SuppressWarnings("serial")
public class DynDialog extends JDialog {

    /**
     * Our listeners should implement dynDialogReact(DynDialog.DynDialogAnswer answer);
     * @author alain
     *
     */
    public interface Listener {
        public void dynDialogReact(Action dynDialogAction);
    }

    public static class Action extends AbstractAction {

        /** @deprecated **/
        private Object caller;
        /** @deprecated **/
        private Method callBack;

        private String actionCommand;
        public String getActionCommand() {
            return actionCommand;
        }

        private Listener localListener;


        public Action(String text, String tooltip, Listener localListener) {
            this(text, (String)null, tooltip);
            this.localListener = localListener;
        }

        public Action(String text, Listener localListener) {
            this(text, (String)null, localListener);
        }

        public Action(String text) { // typically useful for a "Cancel" button
            this(text, (String)null, (Listener)null);
        }

        public Action(String text, String actionCommand, String tooltip) {
            super();
            this.actionCommand = actionCommand;
            putValue(NAME, text);  // => component.setText()
            putValue(SHORT_DESCRIPTION, tooltip);
            putValue(ACTION_COMMAND_KEY, actionCommand); // => components.setActionCommand()
        }
        public Action(String text, String actionCommand) {
            this(text, actionCommand, (String)null);
        }
        /** @deprecated **/
        public Action(String text, String actionCommand, Object caller, Method callBack) {
            this(text, actionCommand, (String)null);
            this.caller = caller;
            this.callBack = callBack;
        }

        public void actionPerformed(ActionEvent e) {

            // Though this clas is static, it can get a pseudo "DynDialog.this" reference to its container.
            // ((JButton)e.getSource()).getTopLevelAncestor() // DynDialog ( <=> "this" )
            DynDialog thisDynDialog = ((DynDialog)
                    ((JButton)
                            e.getSource()
                    ).getTopLevelAncestor()
            );

            // store the action that triggered this handler
            thisDynDialog.lastAction = DynDialog.Action.this;

            // Hide/close this modal dialog
            thisDynDialog.setVisible(false);

            // invoke local, action-specific listener.
            if (this.localListener != null) {
                this.localListener.dynDialogReact(this);
            }

            // invoke local listener, if any (old, deprecated way)
            if ((this.caller != null) && (this.callBack != null)) {
                try {
                    callBack.invoke(caller, (Object[]) null );
                } catch (IllegalArgumentException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                } catch (IllegalAccessException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                } catch (InvocationTargetException e1) {
                    // TODO Auto-generated catch block
                    e1.printStackTrace();
                }
            }

            // invoke global listener, if any
            DynDialog.Listener l = thisDynDialog.listener;
            if (l != null) {
                l.dynDialogReact(this);
            }

        }
    }

    private ArrayList<Action> dynDialogActions;
    private DynDialog.Action lastAction;
    public DynDialog.Action getLastAction() {
        return lastAction;
    }
    /** @deprecated **/
    public boolean addAction(Action dynDialogAction) {
        return dynDialogActions.add(dynDialogAction);
    }
    /** @deprecated **/
    public boolean addAction(String text, String tooltip, String actionCommand) {
        return dynDialogActions.add(new Action(text, actionCommand, tooltip));
    }
    /** @deprecated **/
    public boolean addAction(String text, String actionCommand) {
        return this.addAction(text, null, actionCommand);
    }
    /** @deprecated **/
    public boolean addAction(String text, String actionCommand, Object caller, Method callBack) {
        Action action = new Action(text, actionCommand, caller, callBack);
        // buttonPane.add(new JButton(action));
        return dynDialogActions.add(action);
    }


    private Listener listener;

    private JPanel buttonPane;
    private String dynDialogTitle;
    private String dynDialogText;

    /**
     * Creates a dialog box. Use this constructor with an ArrayList of actions.
     *
     * @param dynDialogTitle Dialog title
     * @param dynDialogText Dialog text
     * @param dynDialogActions An ArrayList of dialog actions
     * @param listener The dialog listener that will be passed this dialog's user actions (a {@link Listener} implementation)
     */
    public DynDialog(String dynDialogTitle, String dynDialogText,
                     ArrayList<DynDialog.Action> dynDialogActions,
                     DynDialog.Listener listener
    ) {
        this.dynDialogTitle = dynDialogTitle;
        this.dynDialogText = dynDialogText;
        this.dynDialogActions = dynDialogActions;
        this.listener = listener;

        this.init();
    }

    /**
     * Creates a dialog box. Use this constructor with an array of actions.
     * @param dynDialogTitle Dialog title
     * @param dynDialogText Dialog text
     * @param dynDialogActions An array of dialog actions
     * @param listener The dialog listener that will be passed this dialog's user actions (a {@link Listener} implementation)
     */
    public DynDialog(String dynDialogTitle, String dynDialogText,
                     DynDialog.Action[] dynDialogActions,
                     DynDialog.Listener listener
    ) {
        this.dynDialogTitle = dynDialogTitle;
        this.dynDialogText = dynDialogText;
        this.dynDialogActions = new ArrayList<DynDialog.Action>();
        for (DynDialog.Action dynDialogAction : dynDialogActions) {
            this.dynDialogActions.add(dynDialogAction);
        }
        this.listener = listener;

        this.init();
    }

    /**
     * Creates a dialog box. Use this constructor to easily create
     * a single-action dialog.
     * @param dynDialogTitle Dialog title
     * @param dynDialogText Dialog text
     * @param dynDialogAction An array of dialog actions
     * @param listener The dialog listener that will be passed this dialog's user actions (a {@link Listener} implementation)
     *
     * @ no more deprecated Un-usable since client code can't create {@link Action} objects outside a DynDialog instance.
     */
    public DynDialog(String dynDialogTitle, String dynDialogText,
                     DynDialog.Action dynDialogAction,
                     DynDialog.Listener listener
    ) {
        this(
                dynDialogTitle,
                dynDialogText,
                new DynDialog.Action[] {dynDialogAction},
                listener
        );
    }


    /**
     * Utility function for constructors; init() creates the GUI.
     */
    private void init() {
        this.setTitle(this.dynDialogTitle);

        this.setBounds(100, 100, 450, 300);
        this.setAlwaysOnTop(true);
        this.setModalityType(DEFAULT_MODALITY_TYPE);
        getContentPane().setLayout(new BorderLayout());

        JPanel contentPanel = new JPanel();
        contentPanel.setLayout(new FlowLayout());
        contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
        getContentPane().add(contentPanel, BorderLayout.CENTER);
        contentPanel.add(new JLabel(this.dynDialogText, null, SwingConstants.LEADING)); // String text, Icon icon, int horizontalAlignment

        buttonPane = new JPanel();
        buttonPane.setLayout(new FlowLayout(FlowLayout.TRAILING));
        for (DynDialog.Action action : this.dynDialogActions) {
            buttonPane.add(new JButton(action));
        }
        getContentPane().add(buttonPane, BorderLayout.SOUTH);

        this.setResizable(false);
        this.pack();
    }


    @Override
    public void setVisible(boolean b) {
        if (b) {
            this.lastAction = null; // reset upon showing
        }
        super.setVisible(b);
    }



}
