package org.lsst.gruth.jutils;

import org.lsst.gruth.grutils.Cst;
import org.lsst.gruth.grutils.Tracer;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.logging.Level;

/**
 * A node for a tree.
 * @author bamade
 */
//TODO: move to pure java by getting rid of Closure with an Interface
public class ComponentNode implements Serializable/*extends Node*/ , Cloneable{
      static class IndirectHash<K,V> extends HashMap<K,V> implements IndirectMap<K,V>{
        public Object getIndirect(Object key) {
            Object obj = get(key);
            if( obj instanceof ComponentNode) {
                obj= ((ComponentNode)obj).getRealValue();
            }
            Tracer.trace(Tracer.NODE_BUILD, "reference to ", key, " " , obj);
            if(obj == null ){
                //TODO : fire error?
                Cst._LOGGER.log(Level.WARNING, "null reference associated to " +key);
            }
            return obj;
        }
    }
    /**
     * A dictionary of all nodes in the tree. name duplications are not detected
     */
    protected /*@NotNull*/ IndirectMap<String, Object> nodeDict;
    /**
     * indicates if this is a root node
     */
    protected boolean root;
    /**
     * indicate if this node is waiting for connection to a tree.
     */
    protected boolean orphan;
    /**
     * a node containing special subnodes that describe method calls to be operated later on object value
     * (may be changed to a special structure)
     */
    protected /*Nullable*/ ComponentNode calls;
    /**
     * the key of the node
     */
    protected /*NotNull*/ String key ;

    // standard NODE functions changed from Node
    protected ComponentNode parent;
    protected String name;
    protected String simpleDescription ;



    protected Object realValue;
    protected Map attributes;
    protected ArrayList<ComponentNode> children;

    public IndirectMap getNodeDict() {
        return this.nodeDict;
    }

    public void setNodeDict(IndirectMap<String, Object> registry) {
        this.nodeDict = registry;
    }

    private void checkDict(ComponentNode parent, Object name, Map attributes) {
        IndirectMap<String, Object> dict;
        if (parent == null) {
            dict = new IndirectHash<String, Object>();
            root = true;
        } else {
            dict = parent.getNodeDict();
        }
        setDict(dict, name, attributes);
    }

    private void setDict(IndirectMap<String, Object> registry, Object name, Map attributes) {
        String keyStr = String.valueOf(name);
        if (attributes != null) {
            Object valName = attributes.get("name");
            if (valName != null) {
                keyStr = String.valueOf(valName);
            }
        }
        this.key = keyStr ;
        //System.out.println("putting :" + key + " " + this.getRealValue());
        registry.put(keyStr, this);
        this.nodeDict = registry;
    }


    public ComponentNode(ComponentNode parent, Object name) {
        //super(parent, name);
        //setDict(parent, name, null);
        this(parent, name, (Map) null, null);
    }

    public ComponentNode(ComponentNode parent, Object name, Object value) {
        //super(parent, name, value);
        //setDict(parent, name, null);
        this(parent, name, (Map) null, value);
    }

    public ComponentNode(ComponentNode parent, Object name, Map attributes) {
        //super(parent, name, attributes);
        // setDict(parent, name, attributes);
        this(parent, name, attributes, null);
    }

    public ComponentNode(ComponentNode parent, Object name, Map attributes, Object value) {
        //super(parent, name, attributes, value);
        this.parent = parent;
        this.simpleDescription = this.name = String.valueOf(name);
        this.attributes = attributes;
        this.realValue = value;
        checkDict(parent, name, attributes);
        //TODO: why on earth does we need to suppres this?
        /*
        if(parent != null ) {
            ArrayList parentChilds = parent.children ;
            if(parentChilds == null ) {
                parentChilds = new ArrayList<ComponentNode>() ;
            }
            System.out.println("ctor adding " + this.getName() + " to " + parent.getName());
            //parentChilds.add(this) ;
        }
        */
    }

    private ComponentNode(IndirectMap<String, Object> registry, Object name, Map attributes, Object value) {
        this.simpleDescription = this.name = String.valueOf(name);
        this.attributes = attributes;
        this.realValue = value;
        setDict(registry, name, attributes);
        orphan = true;
    }


    public ComponentNode clone() {
        Map newAttributes = attributes ;
        if(attributes instanceof MapArgs) {
            newAttributes = ((MapArgs)attributes).clone() ;
        }
        //ComponentNode res = new ComponentNode(parent,name, newAttributes, realValue);
        ComponentNode res = new ComponentNode((ComponentNode) null ,name, newAttributes, realValue);
        //bug here the top node is not with the correct key so we need to change it!

        res.setSimpleDescription(simpleDescription);
        if(children != null) {
            for(ComponentNode child : children) {
                ComponentNode newChild = child.clone() ;
                newChild.parent = null ;
                res.addChild(newChild);
            }
        }
        if (calls != null) res.calls = calls.clone() ;

        return res ;
    }

    public Object calls(Object callDescription) {
        CallBuilders.CallBuilds builder = CallBuilders.getCallBuilder() ;
        this.calls  = builder.buildCalls(this, callDescription) ;
        return this ;
    }

    public ComponentNode setCalls(ComponentNode callsNode) {
        this.calls = callsNode ;
        return this ;
    }

    /*
    public Object calls(Closure closure) {
        ComponentBuilder builder = new ComponentBuilder();
        Object res = builder.invokeMethod("__calls__", closure);
        //this.append((Node) res) ;
        this.calls = (ComponentNode) res;
        //System.out.println("calling calls");
        return this;
    }
    */

    public Object doInvoke(String methodName, Object... args) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        Object support = this.getRealValue();
        return invocation(support,methodName, args) ;

    }

    private Object invocation(Object support, String methodName, Object... args) throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        if(support != null ) {
            /*
             */
            //System.out.println("invocation:" + methodName + Arrays.toString(args));
            return MethodsCalls.invokeMethod(support, methodName, args) ;
            /*
            Class clazz = support.getClass() ;
            Class[] tbargs = new Class[args.length] ;
            for(int ix= 0 ; ix < args.length ; ix++) {
                tbargs[ix] = args[ix].getClass() ;
            }
            Method method = clazz.getMethod(methodName, tbargs) ;
            return method.invoke(support, args) ;
            */
        }
        return null ;
    }

    public Object invokeOn(String objectName, String methodName,Object... args ) throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        Object  obj = this.getIndirect(objectName) ;
        return invocation(obj,methodName, args) ;
    }

    public void doCalls() throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        //System.out.println("--------< do calls : " +this.getName());
        Object curObj = this.getRealValue();
        if (curObj == null) return ;
        if( this.calls != null) {
            ComponentNode topCalls = this.calls ;
            List<ComponentNode> children = topCalls.getChildren();
            if(children == null) {
                System.out.println("empty calls node");
                return ;
            }
            for (ComponentNode child : children) {
                String methodName = child.getName();
                Map attrib = child.getAttributes() ;
                if(attrib == null ) {
                    invocation(curObj, methodName);
                } else {
                    MapArgs mapArgs = (MapArgs) attrib;
                    if(this.nodeDict instanceof IndirectMap) {
                        //System.out.println("mapArgs" + mapArgs);
                        Object[] args = mapArgs.asFullArray(this.nodeDict) ;
                        invocation(curObj, methodName, args);
                    }
                }
            }
        }
    }

    public void doCallsFromTop() throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        doCalls() ;
        ArrayList<ComponentNode> children = this.getChildren() ;
        if(children == null) return ;
        for(ComponentNode child : children) {
            child.doCallsFromTop();
        }
    }

    public void doCallsFromBase() throws InvocationTargetException, NoSuchMethodException, IllegalAccessException {
        for(ComponentNode child : this.getChildren()) {
            child.doCalls();
        }
        doCalls() ;
    }

    public static ComponentNode createOrphan(IndirectMap<String, Object> registry, Object name) {
        return new ComponentNode(registry, name, (Map) null, null);
    }

    public static ComponentNode createOrphan(IndirectMap<String, Object> registry, Object name, Object value) {
        return new ComponentNode(registry, name, (Map) null, value);
    }

    public static ComponentNode createOrphan(IndirectMap<String, Object> registry, Object name, Map attributes) {
        return new ComponentNode(registry, name, attributes, null);
    }

    public static ComponentNode createOrphan(IndirectMap<String, Object> registry, Object name, Map attributes, Object value) {
        return new ComponentNode(registry, name, attributes, value);
    }

    public boolean isRoot() {
        return root;
    }

    public void setRoot(boolean isRoot) {
        root = isRoot ;
    }
    public boolean isOrphan() {
        return orphan;
    }

    public ComponentNode getCalls() {
        return calls;
    }

    public ComponentNode getParent() {
        return parent;
    }

    public String getName() {
        return name;
    }
    public String getSimpleDescription() {
        return simpleDescription;
    }

    public void setSimpleDescription(String simpleDescription) {
        this.simpleDescription = simpleDescription;
    }
    public Object getRealValue() {
        return realValue;
    }

    public Object getNodeByName(String name) {
            //System.out.println("keys" + nodeDict.keySet());
            return this.nodeDict.get(name);
    }

    public Object getIndirect(Object key) {
            Object node = this.nodeDict.get(key);
            if(node instanceof ComponentNode) {
                return ((ComponentNode)node).getRealValue();
            }
           return node ;
    }

    public Iterator<String> getAllKeys() {
            return nodeDict.keySet().iterator() ;
    }

    public Map getAttributes() {
        return attributes;
    }

    public String getKey() {
        return key;
    }

    /**
     *
     * @param registry
     * @param children
     * @throws NullPointerException
     * @throws IllegalArgumentException
     */
    public void addOrphans(/*NotNull*/IndirectMap<String, Object> registry, /*NotNull*/ArrayList<ComponentNode> children)
                        throws  NullPointerException, IllegalArgumentException{
        for (ComponentNode child : children) {
            child.orphan = false;
            if (child.parent == null) {
                //TODO: verify inconsitencies between keys!
                if (child.nodeDict != registry) {
                    registry.putAll(child.getNodeDict());
                    child.nodeDict = registry;
                }
            } else {
                throw new IllegalArgumentException("child added but is part of another node tree");
            }
            child.root = false;
        }
        if (this.children == null) {
            this.children = children;
        } else {
            this.children.addAll(children);
        }
    }

    /**
     *
     * @return the list of child nodes, <TT>null</TT> if there aren't any
     */
    public /*Nullable*/ ArrayList<ComponentNode> getChildren() {
        return children;
    }

    /**
     * adds a ComponentNode as a child of the current node
     * @param child to be added to the node
     * @throws NullPointerException if child is null
     */
    public void addChild(/*NotNull*/ ComponentNode child) throws NullPointerException {
        if (this.children == null) {
            this.children = new ArrayList<ComponentNode>();
        }
        // is this a root or a pseudo-root
        if (child.parent == null) {
            child.root = false;
            if (child.nodeDict != null && child.nodeDict != this.nodeDict) {
                this.nodeDict.putAll(child.getNodeDict());
                child.nodeDict = this.nodeDict;
            } else {//add child to dict
                child.setDict(this.nodeDict, child.getName(), child.getAttributes());
            }
        } else if (child.parent == this) {
            //System.out.println(" parent already set");
            //return ;
        } else {
            throw new IllegalArgumentException("child added but is part of another node tree");
        }
        //System.out.println("adding " + child.getName() + " to " + this.getName());
        this.children.add(child);

    }

    public /*NotNull*/ String toString() {
        return String.valueOf(name) + "("
                + String.valueOf(realValue) + ","
                + String.valueOf(attributes) + ") {"
                + String.valueOf(children) + "}";
    }
}
