package org.lsst.gruth.reports;

import org.lsst.gruth.grutils.AnnotatedReport;
import org.lsst.gruth.grutils.Modification;
import org.lsst.gruth.grutils.RawDiagnostic;
import org.lsst.gruth.grutils.TztReport;

import java.util.TreeMap;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author bamade
 */
// Date: 05/09/12

public class AutomaticModifier {
    StoreAgent store;
    String bundle;
    String versionString;
    int lastVersion;
    int previousVersion;
    TreeMap<String, ComparedAnnotation> lastTree = new TreeMap<String, ComparedAnnotation>();
    TreeMap<String, ComparedAnnotation> previousTree = new TreeMap<String, ComparedAnnotation>();
    int warnings;
    int errors;
    Logger logger = Logger.getLogger(Logger.GLOBAL_LOGGER_NAME);

    public static class ComparedAnnotation {
        boolean compared;
        final AnnotatedReport annotatedReport;

        public ComparedAnnotation(AnnotatedReport annotatedReport) {
            this.annotatedReport = annotatedReport;
        }

        public String getUniqueKey() {
            return this.annotatedReport.getReport().getUniqueKey();
        }
    }

    public AutomaticModifier(StoreAgent store) {
        this.store = store;
    }

    public void compareAndModify(String bundleName, String versionString) throws Exception {
        warnings = 0;
        errors = 0;
        if (versionString == null) {
            // TODO: a bit too Strong: modify interface! to get just latest version number
            lastVersion = store.getVersionNumberFor(bundleName);
            previousVersion = store.getVersion() - 1;
        } else {
            lastVersion = Integer.parseInt(versionString);
            previousVersion = lastVersion - 1;
            /// store.setBundle(bundle,lastVersion);
        }
        /// reads the previousVersion
        store.setBundle(bundle, previousVersion);
        previousTree.clear();
        int maxSizePrevious = store.size();
        for (int ix = 0; ix < maxSizePrevious; ix++) {
            AnnotatedReport aReport = store.get(ix);
            ComparedAnnotation comparedAnnotation = new ComparedAnnotation(aReport);
            previousTree.put(comparedAnnotation.getUniqueKey(), comparedAnnotation);
        }
        /// reads  and modifies the last version
        store.setBundle(bundleName, lastVersion);
        lastTree.clear();
        int maxSizeLast = store.size();
        for (int ix = 0; ix < maxSizeLast; ix++) {
            AnnotatedReport aReport = store.get(ix);
            ComparedAnnotation comparedAnnotation = new ComparedAnnotation(aReport);
            lastTree.put(comparedAnnotation.getUniqueKey(), comparedAnnotation);
        }
        // compare and modify
        for (ComparedAnnotation comparedAnnotation : lastTree.values()) {
            ComparedAnnotation otherAnnotation = previousTree.get(comparedAnnotation.getUniqueKey());
            if (otherAnnotation != null) {
                AnnotatedReport lastAnnotated = comparedAnnotation.annotatedReport;
                AnnotatedReport previousAnnotated = otherAnnotation.annotatedReport;
                lastAnnotated.automaticModification(previousAnnotated, lastVersion, previousVersion);
                // count warning or error
                Modification modification = lastAnnotated.getModification();
                if (modification == null || modification == Modification.CONFUSING_COMPARISON) {
                    TztReport report = lastAnnotated.getReport() ;
                    RawDiagnostic diagnostic = report.getRawDiagnostic() ;
                    if(diagnostic.compareTo(RawDiagnostic.OK_MARK ) <0 ) {
                        logger.log(Level.SEVERE, report.getUniqueKey() + " " +
                                (modification!= null? modification: "" )+ " " + report.getRawDiagnostic()) ;
                        errors++ ;
                    } else if (diagnostic.compareTo(RawDiagnostic.OK_DONE_MARK)< 0 ) {
                        logger.log(Level.WARNING, report.getUniqueKey() + " " +
                        (modification!= null? modification: "" )+ " " + report.getRawDiagnostic()) ;
                        warnings++ ;
                    } else { // confused by correct result
                        if( modification == Modification.CONFUSING_COMPARISON) {
                            logger.log(Level.WARNING, report.getUniqueKey() + " " +
                                    (modification!= null? modification: "" )+ " " + report.getRawDiagnostic()) ;
                            warnings++ ;
                        }
                    }

                } else {
                    switch (modification.getImportance()) {
                        case WARNING:
                            //use logger
                            String message = comparedAnnotation.getUniqueKey() + " " + lastAnnotated.getModification() + " "
                                    + lastAnnotated.getAdvice();
                            logger.log(Level.WARNING, message);
                            warnings++;
                            break;
                        case ERROR:
                            //use logger
                            String messageErr = comparedAnnotation.getUniqueKey() + " " + lastAnnotated.getModification() + " "
                                    + lastAnnotated.getAdvice();
                            logger.log(Level.SEVERE, messageErr);
                            errors++;
                            break;
                    }
                }
                // mark as beenthrough annotated
                lastAnnotated.setBeenThroughAutomaticComparison(true);
                comparedAnnotation.compared = true;
                otherAnnotation.compared = true;
            }
        }
        // the previous is not stored but warn if there are some reports that have not been compared!
        for (ComparedAnnotation comparedAnnotation : previousTree.values()) {
            if (!comparedAnnotation.compared) {

                String message = comparedAnnotation.getUniqueKey() + " lost with this new test";
                logger.log(Level.WARNING, message);
                warnings++;
            }
        }
        // restore last!
        store.save();

        if(warnings > 0  || errors > 0) {
            System.out.println(" warnings : " + warnings + "\n errors : " + errors);
            //TODO System.exit(-1)
        }
    }
}
