package org.lsst.gruth.reports;

import org.lsst.gruth.grutils.AnnotatedReport;
import org.lsst.gruth.grutils.ResultReporter;
import org.lsst.gruth.jutils.NumberedStringGenerator;

import java.io.*;
import java.util.ArrayList;

/**
 * operates on AnnotatedReport
 *
 * @author bamade
 */
// Date: 30/08/12

public class FileStore extends ResultReporter implements StoreAgent {
    public static final String RAW_EXTENSION = ".rawres" ;
    // WAIT : public static final String EDITED_EXTENSION = ".edtres" ;
    private String bundleName;
    private String realName; // with number
    private ArrayList<AnnotatedReport> list = new ArrayList<AnnotatedReport>();
    private int version ;

    public FileStore(String bundleName) throws Exception{
        setBundle(bundleName);
    }

    public FileStore() {

    }

    public int getVersionNumberFor(String bundleName) {
        File dir = new File(".");
        NumberedStringGenerator generator = new NumberedStringGenerator(bundleName+"#"+RAW_EXTENSION);
        for (String fileName : dir.list()) {
            generator.submit(fileName);
        }
        return generator.getMax() ;
    }

    public void setBundle(String bundleName) throws Exception {
        setBundleName(bundleName);
        openFile(realName);
    }

    private void setBundleName(String bundleName) {
        list.clear();
        this.bundleName = bundleName;
        //TODO :mutualize with getversionNUmberFor
        File dir = new File(".");
        NumberedStringGenerator generator = new NumberedStringGenerator(bundleName+"#"+RAW_EXTENSION);
        for (String fileName : dir.list()) {
            generator.submit(fileName);
        }
        realName = generator.getMaxString();
        version = generator.getMax() ;
    }

    public void setBundle(String bundleName, int version) throws Exception {
        realName = bundleName + version + RAW_EXTENSION ;
        this.version = version ;
        openFile(realName);
    }

    public synchronized void openFile(String realName) throws Exception {
        // ouvre
        FileInputStream is = new FileInputStream(realName);
        ObjectInputStream ois = new ObjectInputStream(new BufferedInputStream(is));
        // lit tout
        try {
            while (true) {
                AnnotatedReport report = (AnnotatedReport) ois.readObject();
                list.add(report);
            }
        } catch (EOFException exc) {/*IGNORE*/}
        //System.out.println("size loaded = " + list.size());
        ois.close();
    }

    @Override
    public String[] listBundles() {
        //TODO: list the bundles in the directory use a pattern XXXXXnumber.rawres
        return new String[0];
    }

    public int getVersion() {
        return version;
    }

    public synchronized int  size() {
        return list.size() ;
    }

    public AnnotatedReport get(int index) {
        return list.get(index);
    }

    public void set(int index, AnnotatedReport report) {
        list.set(index,report) ;
    }

    /*
    public FileStore dup() {
        FileStore res = new FileStore() ;
        //TODO: see names
        res.bundleName = bundleName ;
        res.realName = realName ;
        res.list = new ArrayList<AnnotatedReport>(list.size()) ;
        for(AnnotatedReport rep: list) {
            res.list.add(rep.clone()) ;
        }
        return res ;
    }
    */

    public void save() throws IOException {
        //TODO : not a real good idea to change ....
        //int idxExtension = realName.lastIndexOf(RAW_EXTENSION) ;
        //String fileName = realName.substring(0,idxExtension)+EDITED_EXTENSION ;
        String fileName = realName ;
        FileOutputStream os = new FileOutputStream(fileName) ;
        ObjectOutputStream  oos = new ObjectOutputStream(new BufferedOutputStream(os)) ;
        for(AnnotatedReport aReport : list){
            oos.writeObject(aReport);
        }
        oos.flush();
        oos.close();
    }


    @Override
    public void copyTo(ResultReporter reporter) {
        reporter.begin();
        for(AnnotatedReport aReport : list){
            reporter.report(aReport) ;
        }
        reporter.end() ;
    }


    @Override
    public Object report(AnnotatedReport report) {
        list.add(report) ;
        return "";  //To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
    public Object begin() {
        return "" ;
    }

    @Override
    public Object end() {
        return "" ;
    }

    /**
     * report objects in the same test batch can be grouped in different places
     * (files for instance).
     * It is up to the implementation to manage this switch: now the next reports
     * are going to be grouped under this name (for instance in a file with this name).
     * It will be guaranteed that an initial name will be furnished by calling code before any call
     * to report.
     * @param name
     * @return
     */
            @Override
    public Object groupReportsUnderName(String name) {
                setBundleName(name);
                return "" ;
    }

    @Override
    public void close() throws IOException {
        save();
    }
}
