package reportsTests;

import org.lsst.gruth.grutils.*;
import org.lsst.gruth.reports.AutomaticModifier;
import org.lsst.gruth.reports.StoreAgent;

import java.io.*;

/**
 * @author bamade
 */
// Date: 10/09/12

public class ModificationTests implements Serializable {
    //CLASSES WITH SERIAL/EQUALS/TOSTRING properties

    public static class Equals_Serial implements Serializable {
        int val ;

        public Equals_Serial(int val) {
            this.val = val;
        }

        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Equals_Serial)) return false;

            Equals_Serial that = (Equals_Serial) o;

            return (val == that.val);
        }
    }

    public static class Equals_NoSerial {
        int val ;

        public Equals_NoSerial(int val) {
            this.val = val;
        }
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Equals_Serial)) return false;

            Equals_Serial that = (Equals_Serial) o;

            return (val == that.val);
        }
        public String toString() {
            return "val = " + val ;
        }

    }
    public static class NoEquals_Serial  implements  Serializable{
        int val ;
        public NoEquals_Serial(int val) {
            this.val = val;
        }
    }

    public static class NoEquals_NoSerial {
        int val ;
        public NoEquals_NoSerial(int val) {
            this.val = val;
        }
    }
    ///////////////////////////////////////////////////////////

    static AnnotatedReport [][] toCompare = {
            //for each: {before, after }
            {
                    new AnnotatedReport(
                            Modification.ACKNOWLEDGED_POSITIVE, Advice.ACKNOWLEDGED_POSITIVE,
                            new TztReport("ALL_IS_OK", RawDiagnostic.SUCCESS, new Equals_Serial(100), null) )
                    ,
                    new AnnotatedReport(
                            null, null,
                            new TztReport("ALL_IS_OK", RawDiagnostic.SUCCESS, new Equals_Serial(100), null)
                    )
            },
            {
                    new AnnotatedReport(
                            Modification.ACKNOWLEDGED_POSITIVE, Advice.ACKNOWLEDGED_POSITIVE,
                            new TztReport("OK_but_results_differ", RawDiagnostic.SUCCESS, new Equals_Serial(100), null) )
                    ,
                    new AnnotatedReport(
                            null, null,
                            new TztReport("OK_but_results_differ", RawDiagnostic.SUCCESS, new Equals_Serial(7), null)
                    )
            },
            {
                    new AnnotatedReport(
                            Modification.ACKNOWLEDGED_POSITIVE, Advice.ACKNOWLEDGED_POSITIVE,
                            new TztReport("Regression1", RawDiagnostic.SUCCESS, new Equals_Serial(100), null) )
                    ,
                    new AnnotatedReport(
                            null, null,
                            new TztReport("Regression1", RawDiagnostic.FAILED, new Equals_Serial(7), null)
                    )
            },

    } ;

    ////////////////////////////////////////////////////////////
    static class BogusStore implements StoreAgent {
        int lastVersion = 7 ;
        int previousVersion = 6 ;
        int currentVersion ;

        @Override
        public void setBundle(String bundleName) throws Exception {

        }

        @Override
        public void setBundle(String bundleName, int version) throws Exception {
            currentVersion = version ;

        }

        @Override
        public int getVersionNumberFor(String bundleName) {
            return lastVersion;
        }

        @Override
        public int getVersion() {
            return lastVersion;
        }

        @Override
        public String[] listBundles() {
            return new String[0];
        }

        @Override
        public AnnotatedReport get(int index) {
            if(currentVersion == lastVersion) {
               return toCompare[index] [1] ;
            } else{
                return toCompare[index] [0] ;
            }
        }

        @Override
        public int size() {
            return toCompare.length;
        }

        @Override
        public void set(int index, AnnotatedReport report) {

        }

        @Override
        public void save() throws Exception {
            SimpleResultReporter reporter = new SimpleResultReporter();
            for (int ix = 0 ; ix < toCompare.length; ix++) {
               reporter.report(toCompare[ix][1])  ;
            }

            OutputStream fout = new FileOutputStream("bogus7.rawres") ;
            ObjectOutputStream foos = new ObjectOutputStream( new BufferedOutputStream(fout)) ;
            for (int ix = 0 ; ix < toCompare.length; ix++) {
                foos.writeObject(toCompare[ix][1]);  ;
            }
            foos.flush();
            foos.close();
        }

        @Override
        public void copyTo(ResultReporter reporter) {

        }
    }

    ///////////////////////////////////////////////////////////
    public static void main(String[] args) throws  Exception {
        AutomaticModifier modifier = new AutomaticModifier(new BogusStore()) ;
        modifier.compareAndModify("bogus", null);

    }
}
